if(!TESTING)
{
    System.Link("StyleSheet", Module.GetResourceUrl("CHH.css"), "text/css");
}

var Controls = null, CommonDialog = null, IO = null, Communication = null, Management = null;
var Window = null, Control = null, RichEditor = null;

function init(completeCallback, errorCallback)
{
    function LoadModulesComplete()
    {
        IO = System.GetModule("IO.js");
        Communication = System.GetModule("Communication.js");
        Management = System.GetModule("Management.js");
        Controls = System.GetModule("Controls.js");
        CommonDialog = System.GetModule("CommonDialog.js");

        Window = System.GetModule("Window.js").Window;
        Control = System.GetModule("Controls.js").Control;
        
        RichEditor = System.GetModule("RichEditor.js").RichEditor;

        _init(completeCallback, errorCallback);
    }

    System.LoadModules(
        LoadModulesComplete,
        errorCallback,
        ["IO.js", "Communication.js", "Management.js", "Window.js", "Controls.js", "CommonDialog.js", "RichEditor.js"]
    );
}

function _init(completeCallback, errorCallback)
{
    try
    {
        //初始化代码，初始化完成后必须调用completeCallback;
        completeCallback();
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}



function dispose(completeCallback, errorCallback)
{
    _dispose(completeCallback, errorCallback);
}

function _dispose(completeCallback, errorCallback)
{
    try
    {
        //卸载代码，卸载完成后必须调用completeCallback;
        completeCallback();
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}

//共享全局变量和函数，在此定义的变量和函数将由该应用程序的所有实例共享
var LastestPath = System.GetFullPath("Home");

function EmptyCallback()
{

}

function ChmHHC(buffer)
{
    var position = 0;

    var RegxTagName = /(<|<\/)([a-zA-Z]+)(\s[\S\s]*|)>/i;
    var RegxAttrs = /([a-zA-Z1-9]+)\s*=\s*\x22([^\x22]+)\x22/ig;

    function ReadTag()
    {
        var tag = {
            Name: "",
            Type: "",
            Attrs: {}
        };
        var res = null;

        while (res == null)
        {
            if (position >= buffer.length) return null;

            while (position < buffer.length && buffer.charAt(position) != '<') position++;
            if (position >= buffer.length) return null;
            var s = position;

            while (position < buffer.length && buffer.charAt(position) != '>') position++;
            if (position >= buffer.length) return null;
            var e = position;

            position++;

            var tag_str = buffer.substr(s, e - s + 1);
            RegxTagName.lastIndex = 0;
            res = RegxTagName.exec(tag_str);
        }

        tag.Name = res[2].toUpperCase();
        tag.Type = res[1] == '<' ? "Begin": "End";

        if (tag.Type == "Begin" && res.length > 3 && res[3] != "")
        {
            RegxAttrs.lastIndex = 0;
            var atrr = null;
            while ((attr = RegxAttrs.exec(res[3])) != null)
            {
                tag.Attrs[attr[1].toLowerCase()] = attr[2];
            }
        }

        return tag;
    }

    var current = null;

    function IsBeginTag(tag, name)
    {
        return tag.Type == "Begin" && tag.Name == name;
    }

    function IsEndTag(tag, name)
    {
        return tag.Type == "End" && tag.Name == name;
    }

    function RenderTag()
    {
        if (current != null && IsBeginTag(current, "LI"))
        {
            var node = {
                NodeType: "LI",
                SubNodes: []
            };

            current = ReadTag();
            if (current != null && IsBeginTag(current, "OBJECT"))
            {
                node.type = current.Attrs["type"];
                current = ReadTag();
                while (current != null && !IsEndTag(current, "OBJECT"))
                {
                    if (IsBeginTag(current, "PARAM"))
                    {
                        node[current.Attrs["name"]] = current.Attrs["value"];
                    }
                    current = ReadTag();
                }

                if (current != null && IsEndTag(current, "OBJECT")) current = ReadTag();
                if (current != null && IsEndTag(current, "LI")) current = ReadTag();

                while(current != null && IsBeginTag(current, "UL"))
                {
                    var ul = RenderTag();
                    if (ul != null)
                    {
						for(var ul_index in ul.Nodes) node.SubNodes.push(ul.Nodes[ul_index]);
                    }
                }
                
                return node;
            }
        }
        else if (current != null && IsBeginTag(current, "UL"))
        {
            var node = {
                NodeType: "UL",
                Nodes: []
            };

            current = ReadTag();
            while (current != null && !IsEndTag(current, "UL"))
            {
                var subNode = RenderTag();
                if (subNode != null) node.Nodes.push(subNode);
            }

            if (current != null)
            {
                current = ReadTag();
                return node;
            }
        }
        else
        {
            current = ReadTag();
        }
        return null;
    }

    var roots = [];

    this.Render = function()
    {
        position = 0;
        current = ReadTag();

        while (current != null)
        {
            var node = RenderTag();
            if (node != null) roots.push(node);
        }
        current = null;
    }

    this.GetNodes = function()
    {
        return roots;
    }
}

function Application()
{
    var CurrentApplication = this;
    var m_MainForm = null;
    
    //应用程序全局对象;
    var m_HHC = null;
    var m_FileName = null;
    
    this.GetHHC = function()
    {
        return m_HHC;
    }
    
    this.Open = function(path)
    {
        m_MainForm.Open(path);
    }    

    this.Start = function(baseUrl)
    {
        //应用程序入口;
        m_MainForm = new MainForm();
        m_MainForm.OnClosed.Attach(
        function()
        {
            CurrentApplication.Dispose();
        });
        m_MainForm.MoveEx("center", 0, 0);
        m_MainForm.Show(true);
        
        if (arguments[1] != undefined) CurrentApplication.Open(arguments[1]);
    }

    this.Terminate = function(completeCallback, errorCallback)
    {
        try
        {
            //应用程序终止，退出系统时用系统调用;
            completeCallback();
        }
        catch (ex)
        {
            errorCallback(new Exception(ex.mame, ex.message));
        }
    }
    function MainForm()
    {
        var This = this;
        var OwnerForm = this;
        
        var config = {"Left":6,"Top":3,"Width":894,"Height":703,"AnchorStyle":Controls.AnchorStyle.Left | Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":true,"HasMaxButton":true,"Resizable":true,"Title":{"InnerHTML":"阅读器"}};
        
    
        Window.call(This, config);
    
        var Base = {
            GetType: This.GetType,
            is: This.is
        };
    
        This.GetType = function() { return "MainForm"; }
        This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
        
        var toolbar1 = new Controls.Toolbar({"Left":1,"Top":1,"Width":880,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"toolbar","Items":[{"Text":"打开","Css":"Image22_OpenFile","Command":"Open"}]});
        
        
        
        toolbar1.OnCommand.Attach(
            function(command)
            {
                if (command == "Open")
                {
                    var selectForm = new OpenDialog();
                
                    selectForm.OnSelected.Attach(
                    function(path)
                    {
                        This.Open(path);
                    });
                    selectForm.ShowDialog(This, 'center', 0, 0);
                }
            }
        )

        
        var m_Browser_Config={"Left":241,"Top":26,"Width":639,"Height":646,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"CHM_Browser"};
        
                m_Browser_Config.BorderWidth = 1;
                m_Browser_Config.Instance = CurrentApplication;
        
        var m_Browser = new Browser(m_Browser_Config);
        
        

        
        var m_Content_Config={"Left":1,"Top":26,"Width":233,"Height":646,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
        
                m_Content_Config.BorderWidth = 1;
                m_Content_Config.Instance = CurrentApplication;
        
        var m_Content = new ContentTree(m_Content_Config);
        
        m_Content.OnClick.Attach(
        function()
        {
            var node = m_Content.GetSelectedNode();
            if (node == null) return;
            var tag = node.GetTag();
            if (tag == null) return null;
            if (tag.Local != undefined)
            {
                m_Browser.Navigate(System.GetUrl(m_FileName + "/" + tag.Local));
            }
        });
    
    
        var m_Task = null;
        if(config.HasMinButton)
        {
            m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
            This.OnClosed.Attach(
                function()
                {
                    Taskbar.RemoveTask(m_Task);
                }
            );
        }
        var m_Splitor = Controls.CreateVerticalSplit(m_Content, m_Browser);
        
        function HideContent()
        {
            if (m_Content.GetVisible())
            {
                m_Content.SetVisible(false);
                m_Splitor.SetVisible(false);
                m_Browser.Move(m_Browser.GetLeft() - m_Content.GetWidth() - m_Splitor.GetWidth(), m_Browser.GetTop());
                m_Browser.Resize(m_Browser.GetWidth() + m_Content.GetWidth() + m_Splitor.GetWidth(), m_Browser.GetHeight());
            }
        }
        
        function ShowContent()
        {
            if (!m_Content.GetVisible())
            {
                m_Content.SetVisible(true);
                m_Splitor.SetVisible(true);
                m_Browser.Move(m_Browser.GetLeft() + m_Content.GetWidth() + m_Splitor.GetWidth(), m_Browser.GetTop());
                m_Browser.Resize(m_Browser.GetWidth() - m_Content.GetWidth() - m_Splitor.GetWidth(), m_Browser.GetHeight());
            }
        }
        
        This.Open = function(path)
        {
            var ext = System.Path.GetFileExtension(path).toUpperCase();
        
            if (ext == ".CHM")
            {
                This.Waiting();
                try
                {
                    IO.File.Open(
                    function(sysFile)
                    {
                        m_FileName = path;
                        var info = System.ParseJson(sysFile.Content);
                        try
                        {
                            if (info.MainPage != "") m_Browser.Navigate(System.GetUrl(m_FileName + "/" + info.MainPage));
                            else m_Browser.Navigate("");
                            This.SetTitle(info.Title == "" ? System.Path.GetFileName(path) : info.Title);
                        }
                        catch(ex)
                        {}
        
                        IO.File.Open(
                        function(file)
                        {
                            This.Completed();
        
                            m_HHC = new ChmHHC(file.Content);
                            m_HHC.Render();
                            m_Content.Load();
                            ShowContent();
                        },
                        function(ex)
                        {
                            This.Completed();
        
                            m_HHC = null;
                            m_Content.Load();
                            HideContent();
                        },
                        path + "/" + (info.HHC == "" ? ".HHC": info.HHC), false, info.Encoding);
                    },
                    function(ex)
                    {
                        This.Completed();
                        CommonDialog.Alert(ex, This);
                    },
                    path + "/#SYSTEM", false, "utf-8");
        
                }
                catch(ex)
                {
                    This.Completed();
                    CommonDialog.Alert(ex, This);
                }
            }
            else
            {
                m_HHC = null;
                m_Content.Load();
                HideContent();
        
                m_Browser.Navigate(System.GetUrl(path));
            }
        }
    }

}



function Browser(config)
{
    var This = this;
    var OwnerForm = this;
    

    
    var width = config.Width, height = config.Height;
    config.Width=200;
    config.Height=200;

    Controls.Frame.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "Browser"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    

    This.Resize(width,height);
}


function ContentTree_DataSource(instance)
{
    this.GetSubNodes = function(callback, node)
    {
        if (instance.GetHHC() == null)
        {
            callback([]);
            return;
        }

        var roots = instance.GetHHC().GetNodes();
        if (node == null)
        {
            var nodes = [];
            if (roots[0].NodeType == "UL")
            {
                for (var i in roots[0].Nodes)
                {
                    var hhc_node = roots[0].Nodes[i];
                    if (hhc_node.NodeType == "LI")
                    {
                        hhc_node.Index = i;
                        var new_node = {
                            Name: i,
                            Text: IsNull(hhc_node.Name, ""),
                            Tag: hhc_node,
                            ImageCss: hhc_node.SubNodes.length > 0 ? "CHH_CONTENTTREE_CION_BOOK": "CHH_CONTENTTREE_CION_PAGE",
                            HasChildren: hhc_node.SubNodes.length > 0
                        };
                        nodes.push(new_node);
                    }
                }
            }
            callback(nodes);
        }
        else
        {
            var nodes = [];
            var tag = node.GetTag();
            if (tag.SubNodes != undefined)
            {
                for (var i in tag.SubNodes)
                {
                    var hhc_node = tag.SubNodes[i];
                    if (hhc_node.NodeType == "LI")
                    {
                        hhc_node.Index = i;
                        var new_node = {
                            Name: i,
                            Text: IsNull(hhc_node.Name, ""),
                            Tag: hhc_node,
                            ImageCss: hhc_node.SubNodes.length > 0 ? "CHH_CONTENTTREE_CION_BOOK": "CHH_CONTENTTREE_CION_PAGE",
                            HasChildren: hhc_node.SubNodes.length > 0
                        };
                        nodes.push(new_node);
                    }
                    else if (hhc_node.NodeType == "UL")
                    {
                        for (var k in hhc_node.Nodes)
                        {
                            var ul_subnode = hhc_node.Nodes[k];

                            var new_node = {
                                Name: i,
                                Text: IsNull(ul_subnode.Name, ""),
                                Tag: ul_subnode,
                                ImageCss: ul_subnode.SubNodes.length > 0 ? "CHH_CONTENTTREE_CION_BOOK": "CHH_CONTENTTREE_CION_PAGE",
                                HasChildren: ul_subnode.SubNodes.length > 0
                            };
                            nodes.push(new_node);
                        }
                    }
                }

            }
            callback(nodes);
        }
    }
}

function ContentTree(config)
{
    var This = this;
    var OwnerForm = this;
    
config.DataSource = new ContentTree_DataSource(config.Instance);
    
    var width = config.Width, height = config.Height;
    config.Width=200;
    config.Height=200;

    Controls.TreeView.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "ContentTree"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    This.Load = function()
    {
     This.Refresh(EmptyCallback);
    }
    

    This.Resize(width,height);
}

function OpenDialog()
{
    var This = this;
    var OwnerForm = this;
    
    var config = {"Left":6,"Top":4,"Width":787,"Height":610,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":false,"HasMaxButton":false,"Resizable":true,"Title":{"InnerHTML":"请选择要打开的文件(*.chm\u002c.html\u002c*.htm)..."}};
    

    Window.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "OpenDialog"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var tab1 = new Controls.SimpleTabControl({"Left":0,"Top":0,"Width":775,"Height":580,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"simple_tab","Tabs":[{"Text":"浏览","Width":80,"ID":"ID000051","IsSelected":true},{"Text":"共享电子书","Width":120,"ID":"ID000052","IsSelected":false}],"BorderWidth":1});
    
    
    
    tab1.OnSelectedTab.Attach(
        function(index,preIndex)
        {
            if (index == 1)
            {
                This.LoadBooks();
            }
        }
    )

    
    var m_FileBrowser_Config={"Left":5,"Top":5,"Width":763,"Height":511,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":tab1.GetPanel(0),"Text":"","BorderWidth":0};
    m_FileBrowser_Config.Filter = /[^\\\/\x22\|\*\:]*(.chm|.html|.htm)/ig;
    
    m_FileBrowser_Config.OnBeginRequest=function()
    {
        m_FileBrowser.GetTopContainer().Waiting();
    }
    
    m_FileBrowser_Config.OnEndRequest=function()
    {
        m_FileBrowser.GetTopContainer().Completed();
    }
    
    var m_FileBrowser = new CommonDialog.FileBrowser(m_FileBrowser_Config);
    
    m_FileBrowser.SetCurrentPath(LastestPath);
    
    m_FileBrowser.OnFileItemDblClick.Attach(
    function()
    {
        var items = m_FileBrowser.GetSelectedItems();
        if (items.length > 0 && items[0].GetFileInfo("Type") == "F")
        {
            This.Close();
            LastestPath = m_FileBrowser.GetCurrentPath();
            This.OnSelected.Call(m_FileBrowser.GetCurrentPath() + "/" + items[0].GetFileInfo("Name"));
        }
    });

    
    var m_BtnOK = new Controls.Button({"Left":702,"Top":522,"Width":64,"Height":26,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":tab1.GetPanel(0),"Text":"打开","Css":"button_default"});
    
    
    
    m_BtnOK.OnClick.Attach(
        function(btn)
        {
            var items = m_FileBrowser.GetSelectedItems();
            if (items.length > 0 && items[0].GetFileInfo("Type") == "F")
            {
                This.Close();
                LastestPath = m_FileBrowser.GetCurrentPath();
                This.OnSelected.Call(m_FileBrowser.GetCurrentPath() + "/" + items[0].GetFileInfo("Name"));
            }
        }
    )

    
    var m_Container = new Controls.Control({"Left":0,"Top":0,"Width":773,"Height":553,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":tab1.GetPanel(1),"Text":"","Css":"CHH_Share_Container"});
    
    m_Container.GetDom().style.overflowX="hidden";
    m_Container.GetDom().style.overflowY="scroll";
    
    m_Container.OnResized.Attach(
        function(btn)
        {
            
        }
    )
    

    var m_Task = null;
    if(config.HasMinButton)
    {
        m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
        This.OnClosed.Attach(
            function()
            {
                Taskbar.RemoveTask(m_Task);
            }
        );
    }
    This.OnSelected = new Delegate();
    
    This.LoadBooks = function()
    {
        This.Waiting("正在读取列表...");
        IO.File.Open(
        function(file)
        {
            This.Completed();
            try
            {
                var books = System.ParseJson(file.Content);
    
                m_Container.Clear();
                var top = 0;
                for (var category in books)
                {
                    var header = new CategoryPanel({
                        Left: 0,
                        Top: top,
                        Width: m_Container.GetClientWidth() - 18,
                        Height: 24,
                        AnchorStyle: Controls.AnchorStyle.Left | Controls.AnchorStyle.Right | Controls.AnchorStyle.Top,
                        Parent: m_Container,
                        Css: "CHH_Share_CategoryPanel"
                    });
                    header.SetText(category);
    
                    top += 24;
    
                    for (var i in books[category])
                    {
                        new BookPanel({
                            Left: 0,
                            Top: top,
                            Width: m_Container.GetClientWidth() - 18,
                            Height: 110,
                            AnchorStyle: Controls.AnchorStyle.Left | Controls.AnchorStyle.Right | Controls.AnchorStyle.Top,
                            Parent: m_Container,
                            Css: "CHH_BookPanel",
                            Info: books[category][i]
                        });
                        top += 110;
                    }
                }
            }
            catch(ex)
            {
                alert(ex);
            }
        },
        function(ex)
        {
            This.Completed();
            alert(ex);
        },
        System.GetSpecialDirectory("System") + "/Books.conf", false, "utf-8");
    }
}




function BookPanel(config)
{
    var This = this;
    var OwnerForm = this;
    

    
    var width = config.Width, height = config.Height;
    config.Width=631;
    config.Height=110;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "BookPanel"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_Image = new Controls.Control({"Left":7,"Top":7,"Width":96,"Height":96,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"control"});
    
    m_Image.GetDom().innerHTML = "<img style='border:solid 0px black;'/>";
    m_Image.GetDom().firstChild.onload = function()
    {
        try
        {
            var img = m_Image.GetDom().firstChild;
            if (img.width > img.height)
            {
                img.style.width = "96px";
                img.style.height = (img.height * 96 / img.width) + "px";
                img.style.marginLeft = "0px";
                img.style.marginTop = Math.round((96 - (img.height * 96 / img.width)) / 2) + "px";
            }
            else
            {
                img.style.height = "96px";
                img.style.width = (img.width * 96 / img.height) + "px";
                img.style.marginTop = "0px";
                img.style.marginLeft = Math.round((96 - (img.width * 96 / img.height)) / 2) + "px";
            }
        }
        catch(ex)
        {
    
        }
    }
    
    m_Image.GetDom().firstChild.src = ((config.Info.Image == undefined || config.Info.Image == "") ? Module.GetResourceUrl("ebook.gif") : System.GetUrl(System.GetSpecialDirectory("Public") + "/" + config.Info.Image));
    
    m_Image.OnResized.Attach(
        function(btn)
        {
            
        }
    )

    
    var m_Title = new Controls.Label({"Left":115,"Top":6,"Width":509,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"HTML语言大全","Css":"CHH_BookPanel_Title"});
    
    m_Title.SetText(IsNull(config.Info.Title, ""));

    
    var m_Summary = new Controls.Label({"Left":114,"Top":25,"Width":510,"Height":58,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"简介","Css":"CHH_BookPanel_Summary"});
    
    m_Summary.SetText(IsNull(config.Info.Summary, ""));

    
    var m_Open = new Controls.Label({"Left":557,"Top":88,"Width":56,"Height":14,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"开始阅读","Css":"CHH_BookPanel_Link"});
    
    m_Open.GetDom().onclick = function()
    {
        var form = This.GetTopContainer();
        form.Close();
        form.OnSelected.Call(System.GetSpecialDirectory("Public") + "/" + config.Info.Path);
    }
    

    

    This.Resize(width,height);
}




function CategoryPanel(config)
{
    var This = this;
    var OwnerForm = this;
    

    
    var width = config.Width, height = config.Height;
    config.Width=672;
    config.Height=24;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "CategoryPanel"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    This.SetText = function(text)
    {
        This.GetDom().innerHTML = String.format("<div>{0}</div>", text);
    }

    This.Resize(width,height);
}

