if(!TESTING)
{
    System.Link("StyleSheet", Module.GetResourceUrl("ChatRoom.css"), "text/css");
}

var Controls = null, CommonDialog = null, IO = null, Communication = null, Management = null;
var Window = null, Control = null, RichEditor = null;

function init(completeCallback, errorCallback)
{
    function LoadModulesComplete()
    {
        IO = System.GetModule("IO.js");
        Communication = System.GetModule("Communication.js");
        Management = System.GetModule("Management.js");
        Controls = System.GetModule("Controls.js");
        CommonDialog = System.GetModule("CommonDialog.js");

        Window = System.GetModule("Window.js").Window;
        Control = System.GetModule("Controls.js").Control;
        
        RichEditor = System.GetModule("RichEditor.js").RichEditor;

        _init(completeCallback, errorCallback);
    }

    System.LoadModules(
        LoadModulesComplete,
        errorCallback,
        ["IO.js", "Communication.js", "Management.js", "Window.js", "Controls.js", "CommonDialog.js", "RichEditor.js"]
    );
}

function _init(completeCallback, errorCallback)
{
    try
    {
        //初始化代码，初始化完成后必须调用completeCallback;
        completeCallback();
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}



function dispose(completeCallback, errorCallback)
{
    _dispose(completeCallback, errorCallback);
}

function _dispose(completeCallback, errorCallback)
{
    try
    {
        //卸载代码，卸载完成后必须调用completeCallback;
        completeCallback();
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}

//共享全局变量和函数，在此定义的变量和函数将由该应用程序的所有实例共享
function EmptyCallback()
{

}

function Application()
{
    var CurrentApplication = this;
    var m_MainForm = null;
    
    //应用程序全局对象;
        

    this.Start = function(baseUrl)
    {
        //应用程序入口;
        if(TESTING)
        {
            var form = new Window(
                {
                    Left: 0, Top: 0, Width: 800, Height: 600,
                    Parent: null,
                    Css: "window", BorderWidth: 6,
                    HasMinButton: true, HasMaxButton: true,
                    Title: {
                        InnerHTML: "调试"
                    }
                }
            );
            
            m_MainForm = new MainWnd(
                {
                    Left: 0, Top: 0, Width: form.GetClientWidth(), Height: form.GetClientHeight(),
                    Parent: form,
                    AnchorStyle: Controls.AnchorStyle.All,
                    Css: "mainwnd"
                }
            );
            
            form.OnClosed.Attach(
                function()
                {
                    CurrentApplication.Dispose();
                }
            );
            
            form.MoveEx("center",0,0);
            form.Show();
            form.Maximun();
        }
        else
        {
            m_MainForm = new MainWnd(
                {
                    Left: 0, Top: 0, Width: Desktop.GetClientWidth(), Height: Desktop.GetClientHeight(),
                    Parent: Desktop,
                    AnchorStyle: Controls.AnchorStyle.All,
                    Css: "mainwnd"
                }
            );
        }
        
        
        
    }

    this.Terminate = function(completeCallback, errorCallback)
    {
        try
        {
            //应用程序终止，退出系统时用系统调用;
            completeCallback();
        }
        catch (ex)
        {
            errorCallback(new Exception(ex.mame, ex.message));
        }
    }
    function MainForm()
    {
        var This = this;
        var OwnerForm = this;
        
        var config = {"Left":0,"Top":0,"Width":400,"Height":300,"AnchorStyle":Controls.AnchorStyle.Left | Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":true,"HasMaxButton":true,"Resizable":true,"Title":{"InnerHTML":"主窗口"}};
        
    
        Window.call(This, config);
    
        var Base = {
            GetType: This.GetType,
            is: This.is
        };
    
        This.GetType = function() { return "MainForm"; }
        This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    
        var m_Task = null;
        if(config.HasMinButton)
        {
            m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
            This.OnClosed.Attach(
                function()
                {
                    Taskbar.RemoveTask(m_Task);
                }
            );
        }
        
    }

}



function MainWnd(config)
{
    var This = this;
    var OwnerForm = this;
    

    
    var width = config.Width, height = config.Height;
    config.Width=604;
    config.Height=483;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "MainWnd"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_FriendPanel_Config={"Left":347,"Top":6,"Width":250,"Height":471,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
    
            m_FriendPanel_Config.Css = "ChatRoom_FirendPanel";
            m_FriendPanel_Config.BorderWidth = 1;
    
    var m_FriendPanel = new FriendPanel(m_FriendPanel_Config);
    
    

    
    var m_ChatRoomTab_Config={"Left":7,"Top":6,"Width":333,"Height":471,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
    
            
    
    var m_ChatRoomTab = new ChatRoomTabCtrl(m_ChatRoomTab_Config);
    
    
    

    Controls.CreateVerticalSplit(m_ChatRoomTab, m_FriendPanel, false);

    This.Resize(width,height);
}


var m_Rooms = {
    "技术交流": [{
        Name: "Javascript_Room",
        Nickname: "Javascript讨论区"
    },
    {
        Name: "CPP_Room",
        Nickname: "C/C++讨论区"
    },
    {
        Name: "dotNet_Room",
        Nickname: ".NET讨论区"
    },
    {
        Name: "Win32_Room",
        Nickname: "Win32讨论区"
    }],
    "其它": [{
        Name: "10000",
        Nickname: "站务讨论"
    }]
};

var FriendPanelDataSource = {};
FriendPanelDataSource.GetSubNodes = function(callback, node)
{
    if (node == null)
    {
        var nodes = [{
            Name: "Room",
            Text: "聊天室",
            Tag: null,
            ImageCss: "Image16_Folder"
        },
        {
            Name: "Users",
            Text: "好友",
            Tag: null,
            ImageCss: "Image16_Folder"
        },
        {
            Name: "Groups",
            Text: "群",
            Tag: null,
            ImageCss: "Image16_Folder"
        }];
        callback(nodes);
    }
    else
    {
        var nodes = [];
        if (node.GetFullPath() == "/Room")
        {
            for (var k in m_Rooms)
            {
                nodes.push({
                    Name: k,
                    Text: k,
                    Tag: m_Rooms[k],
                    ImageCss: "Image16_Folder"
                });
            }
        }
        else
        {
            var tag = node.GetTag();
            for (var i in tag)
            {
                nodes.push({
                    Name: tag[i].Name,
                    Text: tag[i].Nickname,
                    Tag: tag[i],
                    ImageCss: "Image16_Group",
                    HasChildren: false
                });
            }
        }
        callback(nodes);
    }
}

function FriendPanel(config)
{
    var This = this;
    var OwnerForm = this;
    
config.DataSource = FriendPanelDataSource;
    
    var width = config.Width, height = config.Height;
    config.Width=226;
    config.Height=588;

    Controls.TreeView.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "FriendPanel"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    This.Refresh(EmptyCallback);
    This.Expand(EmptyCallback, "/Room/技术交流");
    This.Expand(EmptyCallback, "/Room/其它");

    This.Resize(width,height);
}




function ChatRoomTabCtrl(config)
{
    var This = this;
    var OwnerForm = this;
    

    
    var width = config.Width, height = config.Height;
    config.Width=548;
    config.Height=537;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "ChatRoomTabCtrl"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var tab1 = new Controls.SimpleTabControl({"Left":0,"Top":0,"Width":548,"Height":537,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"simple_tab","Tabs":[{"Text":"标签1","Width":80,"IsSelected":true,"ID":"ID000055"},{"Text":"标签2","Width":80,"IsSelected":false,"ID":"ID000056"}],"BorderWidth":1});
    
    
    
    tab1.OnSelectedTab.Attach(
        function(index,preIndex)
        {
            
        }
    )

    
    var customcontrol2_Config={"Left":5,"Top":5,"Width":536,"Height":500,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":tab1.GetPanel(0),"Text":"","Css":"control"};
    
            
    
    var customcontrol2 = new ChatRoomCtrl(customcontrol2_Config);
    
    
    

    

    This.Resize(width,height);
}




function MessagePanel(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "MessagePanel";
config.BorderWidth=1;
    
    var width = config.Width, height = config.Height;
    config.Width=540;
    config.Height=403;

    Controls.Frame.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "MessagePanel"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    This.Link("StyleSheet", Module.GetResourceUrl("EditorCss.css"), "text/css");var m_FileHtmlFormat = "<div class='div_filename'>文件名:{0}</div>" + "<div class='operationContainer'>" + "<div class='link_download'><a target='_blank' href='download.los?FileName={1}'>下载</a></div>" + "<div class='link_saveas'><span onclick='javascript:window.SaveFile(unescape(\"{1}\"))'>另存为</span></div>" + "<div class='link_open'><span onclick='javascript:window.OpenFile(unescape(\"{1}\"))'>打开</span></div>" + "</div>";
    
    This.OnCommand = new Delegate();
    
    This.Clear = function()
    {
        This.GetDocument().body.innerHTML = "";
    }
    
    This.CreateMessage = function(content)
    {
        content = content.replace(/<img\s[^<>]*>/ig,
        function(val)
        {
            return String.format("<img src='{0}'/>", Module.GetResourceUrl("loading_img.gif"));
        });
    
        content = content.replace(/\x5BFILE\x3A\s*\{Accessory\s+type=\"file\"\s+src=\"([^\t\n\r\f\v\x5B\x5D]+)\"\}\x5D/ig,
        function(val, file)
        {
            return String.format("<div class='div_filename'><img src='{0}'/></div>", Module.GetResourceUrl("loading_file.gif"), file);
        });
    
        var msgDiv = This.GetDocument().createElement("DIV");
        msgDiv.className = "message";
        msgDiv.innerHTML = String.format("<div class='messageTitle'>" + "<span class='sender'>{0}</span>" + "<span class='time'>{1}</span>" + "<div class='operationContainer' style='display:none;'>" + "</div>" + "</div>" + "<div class='messageContent'>{2}" + "</div>", Me.Nickname, DateToString(new Date()), content);
        This.GetDocument().body.appendChild(msgDiv);
        msgDiv.scrollIntoView();
        return msgDiv;
    }
    
    This.ShowError = function(msg, temp)
    {
        temp.childNodes[1].innerHTML =String.format("<span style='color:Red; '>{0}</span>", System.ReplaceHtml(msg));
    }
    
    This.AddMessage = function(msg, temp)
    {
        msg.Content = msg.Content.replace(/\x5BFILE\x3A[^\t\n\r\f\v\x5B\x5D]+\x5D/ig,
        function(val)
        {
            var file = val.substr(6, val.length - 7);
            return String.format(
            m_FileHtmlFormat, System.Path.GetFileName(unescape(file)), file);
        });
    
        msg.Content = msg.Content.replace(/\x5BA\x3A[^\t\n\r\f\v\x5B\x5D]+\x5D/ig,
        function(val)
        {
            var file = val.substr(3, val.length - 4);
            return String.format("<a target='_blank' href='{0}' onclick='return window.OpenLink(this.href);'>", unescape(file));
        });
    
        msg.Content = msg.Content.replace(/\x5B\x2FA\x5D/ig,
        function(val)
        {
            return "</a>";
        });
    
        var msgDiv = This.GetDocument().createElement("DIV");
        msgDiv.className = "message";
        msgDiv.innerHTML = String.format("<div class='messageTitle'>" + "<span class='sender'>{0}</span>" + "<span class='time'>{1}</span>" + "<div class='operationContainer' style='display:none;'>" + "</div>" + "</div>" + "<div class='messageContent'>{2}" + "</div>", msg.SenderNickname, DateToString(msg.CreatedTime), msg.Content);
        msgDiv.onmousemove = function()
        {
            this.firstChild.childNodes[2].style.display = "inline";
        }
        msgDiv.onmouseout = function()
        {
            this.firstChild.childNodes[2].style.display = "none";
        }
    
        if (config.Operation != undefined)
        {
            for (var i in config.Operation)
            {
                try
                {
                    var ope = config.Operation[i];
                    (function(text, cmd, msg, msgHtml, callback)
                    {
                        var opeLink = This.GetDocument().createElement("A");
                        opeLink.className = "operation";
                        opeLink.innerHTML = text;
                        opeLink.onclick = function()
                        {
                            callback(cmd, msg, msgHtml);
                        }
                        var opeContainer = msgDiv.firstChild.childNodes[2];
                        opeContainer.appendChild(opeLink);
                    })(ope.Text, ope.Command, msg, msgDiv.innerHTML,
                    function(cmd, msg, msgHtml)
                    {
                        This.OnCommand.Call(cmd, msg, msgHtml);
                    });
                }
                catch(ex)
                {}
            }
        }
        if (temp == undefined)
        {
            This.GetDocument().body.appendChild(msgDiv);
        }
        else
        {
            This.GetDocument().body.replaceChild(msgDiv, temp);
        }
        msgDiv.scrollIntoView();
        msgDiv = null;
    }
    
    This.GetWindow().OpenLink = function(href)
    {
        return confirm(String.format("您确定要打开链接 \"{0}\"", href));;
    }
    
    This.GetWindow().SaveFile = function(filePath)
    {
        var folderBrowser = new CommonDialog.FolderBrowserDialog({
            Left: 0,
            Top: 0,
            Width: 400,
            Height: 400,
            Css: 'window',
            BorderWidth: 6,
            Resizable: true,
            HasMinButton: false,
            Title: {
                Height: 18,
                InnerHTML: '另存为'
            },
            Desription: "请选择文件夹:",
            Root: System.GetFullPath("Home")
        });
        folderBrowser.OnSelected.Attach(
        function(fb)
        {
            var path = fb.GetSelectedFolderPath();
            if (path != null)
            {
                This.GetTopContainer().Waiting("正在保存文件...");
    
                IO.File.Copy(
                function()
                {
                    This.GetTopContainer().Completed();
                },
                function(error)
                {
                    This.GetTopContainer().Completed();
                    CommonDialog.Alert(error.Message, This.GetTopContainer());
                },
                [filePath], path);
            }
        });
        folderBrowser.ShowDialog(This.GetTopContainer(), 'center', 0, 0);
        folderBrowser.Focus();
    }
    
    This.GetWindow().OpenFile = function(filePath)
    {
        System.Exec(function()
        {},
        alert, "OpenWith.js", filePath, false);
    }

    This.Resize(width,height);
}




function MessageEditor(config)
{
    var This = this;
    var OwnerForm = this;
    
config.BorderWidth = 1;
    
    var width = config.Width, height = config.Height;
    config.Width=384;
    config.Height=202;

    RichEditor.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "MessageEditor"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    

    This.Resize(width,height);
}




function ChatRoomCtrl(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "ChatRoomCtrl";
    
    var width = config.Width, height = config.Height;
    config.Width=489;
    config.Height=479;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "ChatRoomCtrl"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_Toolbar = new Controls.Toolbar({"Left":0,"Top":0,"Width":489,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"toolbar","Items":[{"Text":"聊天记录","Css":"Image22_MessageRecord","Command":"ViewRecord"},{"Text":"退出聊天室","Css":"Image22_Delete","Command":"Exit"}]});
    
    
    
    m_Toolbar.OnCommand.Attach(
        function(command)
        {
            
        }
    )

    
    var m_MessagePanel_Config={"Left":0,"Top":27,"Width":489,"Height":247,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
    
            
    
    var m_MessagePanel = new MessagePanel(m_MessagePanel_Config);
    
    

    
    var m_MessageEditor_Config={"Left":0,"Top":281,"Width":489,"Height":164,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"};
    
            m_MessageEditor_Config.StyleSheet = Module.GetResourceUrl("EditorCss.css");
    
    var m_MessageEditor = new MessageEditor(m_MessageEditor_Config);
    
    

    
    var button4 = new Controls.Button({"Left":422,"Top":451,"Width":65,"Height":26,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"发送","Css":"button_default"});
    
    
    
    button4.OnClick.Attach(
        function(btn)
        {
            
        }
    )
    

    Controls.CreateHorizSplit(m_MessagePanel, m_MessageEditor, false);

    This.Resize(width,height);
}

