if(!TESTING)
{
    System.Link("StyleSheet", Module.GetResourceUrl("Finance.css"), "text/css");
}

var Controls = null, CommonDialog = null, IO = null, Communication = null, Management = null;
var Window = null, Control = null, RichEditor = null;

function init(completeCallback, errorCallback)
{
    function LoadModulesComplete()
    {
        IO = System.GetModule("IO.js");
        Communication = System.GetModule("Communication.js");
        Management = System.GetModule("Management.js");
        Controls = System.GetModule("Controls.js");
        CommonDialog = System.GetModule("CommonDialog.js");

        Window = System.GetModule("Window.js").Window;
        Control = System.GetModule("Controls.js").Control;
        
        RichEditor = System.GetModule("RichEditor.js").RichEditor;

        _init(completeCallback, errorCallback);
    }

    System.LoadModules(
        LoadModulesComplete,
        errorCallback,
        ["IO.js", "Communication.js", "Management.js", "Window.js", "Controls.js", "CommonDialog.js", "RichEditor.js"]
    );
}

function _init(completeCallback, errorCallback)
{
    try
    {
        //初始化代码，初始化完成后必须调用completeCallback;
        System.LoadModules(
        function()
        {
            SQLite = System.GetModule("Data.js").SQLite;
            completeCallback();
        },
        errorCallback, ["Data.js"]);
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}



function dispose(completeCallback, errorCallback)
{
    _dispose(completeCallback, errorCallback);
}

function _dispose(completeCallback, errorCallback)
{
    try
    {
        //卸载代码，卸载完成后必须调用completeCallback;
        completeCallback();
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}

//共享全局变量和函数，在此定义的变量和函数将由该应用程序的所有实例共享
var Config = {
    Default:System.GetFullPath("Config/Finance/default.db")
};

function EmptyCallback()
{}

function Round(amount)
{
    return Math.round(amount*100)/100;
}

function GetDayCount(year, month)
{
    var days = [31, ((year % 4 == 0 && (year % 100 != 0 || year % 400 == 0)) ? 29 : 28), 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    return days[month - 1];
}

function HandleTabKey(ctrl, next, handler)
{
    System.AttachEvent(
    ctrl, "keydown",
    function(evt)
    {
        if (evt == undefined) evt = window.event;
        if (evt.keyCode == 9 && !evt.shiftKey && !evt.ctrlKey && !evt.altKey)
        {
            if (next != null) next.focus()
            else handler();
            System.PreventDefault(evt);
        }
    });
}

function HandleKey(keyCode, ctrl, next, handler)
{
    System.AttachEvent(
    ctrl, "keydown",
    function(evt)
    {
        if (evt == undefined) evt = window.event;
        if (evt.keyCode == keyCode && !evt.shiftKey && !evt.ctrlKey && !evt.altKey)
        {
            if (next != null) next.focus()
            else handler();
            System.PreventDefault(evt);
        }
    });
}


function TranslateCharForSQL(str)
{
    return str.replace(/\'/ig,
    function()
    {
        return "''";
    });
}

//格式化数字
function FormatNumber(num, length)
{
    var s = num.toString();
    var zero = "";
    for (var i = 0; i < length - s.length; i++) zero += "0";
    return zero + s;
}

//时间转字符串
function DateToString(date)
{
    return String.format("{0}-{1}-{2}", FormatNumber(date.getFullYear(), 4), FormatNumber(date.getMonth() + 1, 2), FormatNumber(date.getDate(), 2), FormatNumber(date.getHours(), 2), FormatNumber(date.getMinutes(), 2), FormatNumber(date.getSeconds(), 2));
}

var SQLite = null;

var DDL =
'Begin Transaction;\r\n'+
'\r\n'+
'Create Table [AccountBookInfo]\r\n'+
'(\r\n'+
'    [Version] TEXT NOT NULL,\r\n'+
'    [ApplicationVersion] TEXT NOT NULL,\r\n'+
'    [Name] TEXT NOT NULL,\r\n'+
'    [Config] TEXT NOT NULL\r\n'+
');\r\n'+
'\r\n'+
'Create Table [FinanceItem]\r\n'+
'(\r\n'+
'    [Key] INTEGER PRIMARY KEY NOT NULL,\r\n'+
'    [Name] TEXT NOT NULL,\r\n'+
'    [Type] INT NOT NULL DEFAULT 0,\r\n'+
'    [Parent] INTEGER NOT NULL DEFAULT 0,\r\n'+
'    [Shutcut] TEXT\r\n'+
');\r\n'+
'\r\n'+
'\r\n'+
'Create Table [FinanceRecord]\r\n'+
'(\r\n'+
'    [Key] BIGINT PRIMARY KEY NOT NULL,\r\n'+
'    [Comment] TEXT,\r\n'+
'    [Amount] FLOAT NOT NULL DEFAULT 0,\r\n'+
'    [Type] INTEGER NOT NULL,\r\n'+
'    [CreatedTime] TIMESTAMP NOT NULL,\r\n'+
'    [LastRenewTime] TIMESTAMP NOT NULL\r\n'+
');\r\n'+
'\r\n'+
'Create Index IDX_FR_TYPE on [FinanceRecord](Type);\r\n'+
'Create Index IDX_FR_CTIME on [FinanceRecord](CreatedTime);\r\n'+
'Create Index IDX_FR_RTIME on [FinanceRecord](LastRenewTime);\r\n'+
'\r\n'+
'insert into [AccountBookInfo] (Version,ApplicationVersion,Name,Config) values (\u00271.0.1.0\u0027,\u00271.0.1.0\u0027,\u0027\u0027,\u0027\u0027);\r\n'+
'\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (1,\u0027工资\u0027,0,\u0027G\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (2,\u0027奖金\u0027,0,\u0027J\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (3,\u0027分红\u0027,0,\u0027F\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (5,\u0027礼金\u0027,0,\u0027L\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (6,\u0027其他收入\u0027,0,\u0027Q\u0027);\r\n'+
'\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (7,\u0027日常开支\u0027,1,\u0027R\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (8,\u0027交通\u0027,1,\u0027T\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (9,\u0027饮食\u0027,1,\u0027Y\u0027);  \r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (10,\u0027通讯\u0027,1,\u0027X\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (11,\u0027娱乐\u0027,1,\u0027Z\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (12,\u0027孝敬父母\u0027,1,\u0027M\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (13,\u0027卫生医疗费\u0027,1,\u0027W\u0027);\r\n'+
'insert into [FinanceItem] (Key,Name,Type,Shutcut) values (14,\u0027其他开支\u0027,1,\u0027C\u0027);\r\n'+
'\r\n'+
'Commit Transaction;\r\n'

var SelectFinanceItemsCmd = "select Key,Type,Name,Parent,Shutcut from FinanceItem";
var InsertFinanceRecordCmd = "insert into FinanceRecord (Key,Type,Comment,Amount,CreatedTime,LastRenewTime) values ({5},{0},'{1}',{2},datetime('{3}','unixepoch','localtime'),datetime('{4}','unixepoch','localtime'))";
var UpdateFinanceRecordCmd = "update FinanceRecord set Type = {0}, Comment = '{1}', Amount = {2}, CreatedTime = datetime('{3}','unixepoch','localtime'), LastRenewTime = datetime('{4}','unixepoch','localtime') where Key = {5}";
var DeleteFinanceRecordCmd = "delete from FinanceRecord where Key = {0}";

function Application()
{
    var CurrentApplication = this;
    var m_MainForm = null;
    
    //应用程序全局对象;
    var m_DbPath = "";
    var m_FinanceItems = {
        Income: [],
        Expense: [],
        All: {},
        Shutcut: {}
    };
    
    this.GetFinanceItems = function()
    {
        return m_FinanceItems;
    }
    
    this.GetFinanceItemDetails = function(key)
    {
        var ret = m_FinanceItems.All[key];
        return ret == undefined ? null: ret;
    }
    
    this.GetFinanceItemByShutcut=function(shutcut)
    {
        var ret = m_FinanceItems.Shutcut[shutcut];
        return ret == undefined ? null: ret;
    }
    
    this.GetDbPath = function()
    {
        return m_DbPath;
    }    

    this.Start = function(baseUrl)
    {
        //应用程序入口;
        m_MainForm = new MainForm();
        m_MainForm.OnClosed.Attach(
            function()
            {
                CurrentApplication.Dispose();
            }
        );
        m_MainForm.MoveEx("center", 0, 0);
        m_MainForm.Show(true);
        
        m_MainForm.Open(Config.Default);
        
    }

    this.Terminate = function(completeCallback, errorCallback)
    {
        try
        {
            //应用程序终止，退出系统时用系统调用;
            completeCallback();
        }
        catch (ex)
        {
            errorCallback(new Exception(ex.mame, ex.message));
        }
    }
    function MainForm()
    {
        var This = this;
        var OwnerForm = this;
        
        var config = {"Left":7,"Top":11,"Width":800,"Height":650,"AnchorStyle":Controls.AnchorStyle.Left | Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":true,"HasMaxButton":true,"Resizable":true,"Title":{"InnerHTML":"日常理财"}};
        config.MinWidth = 750;
        config.MinHeight = 550;
    
        Window.call(This, config);
    
        var Base = {
            GetType: This.GetType,
            is: This.is
        };
    
        This.GetType = function() { return "MainForm"; }
        This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
        
        var m_Tab = new Controls.SimpleTabControl({"Left":1,"Top":26,"Width":786,"Height":593,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"simple_tab","Tabs":[{"Text":"新增记录","Width":100,"ID":"ID000036","IsSelected":true},{"Text":"我的账本","Width":100,"ID":"ID000037","IsSelected":false}],"BorderWidth":1});
        
        
        
        m_Tab.OnSelectedTab.Attach(
            function(index,preIndex)
            {
                
            }
        )

        
        var m_RecordGrid_Config={"Left":1,"Top":110,"Width":782,"Height":455,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":m_Tab.GetPanel(0),"Text":"","Css":"control"};
        
                m_RecordGrid_Config.BorderWidth = 0;
                m_RecordGrid_Config.Application = CurrentApplication;
        
        var m_RecordGrid = new FinanceRecordGrid(m_RecordGrid_Config);
        
        

        
        var m_NewRecordPanel = new Controls.Control({"Left":4,"Top":8,"Width":773,"Height":100,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":m_Tab.GetPanel(0),"Text":"","Css":"control"});
        
        
        
        m_NewRecordPanel.OnResized.Attach(
            function(btn)
            {
                
            }
        )

        
        var label49 = new Controls.Label({"Left":5,"Top":7,"Width":34,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_NewRecordPanel,"Text":"日期","Css":"FinanceLabeBold"});
        
        

        
        var m_NewDate_Config={"Left":44,"Top":4,"Width":177,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_NewRecordPanel,"Text":"","Css":"control"};
        
                
        
        var m_NewDate = new DateControl(m_NewDate_Config);
        
        

        
        var label51 = new Controls.Label({"Left":238,"Top":7,"Width":60,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_NewRecordPanel,"Text":"收支项目","Css":"FinanceLabeBold"});
        
        

        
        var m_NewItem_Config={"Left":302,"Top":4,"Width":280,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":m_NewRecordPanel,"Text":"","Css":"control"};
        
                m_NewItem_Config.Application = CurrentApplication;
        
        var m_NewItem = new FinanceItemControl(m_NewItem_Config);
        
        

        
        var label53 = new Controls.Label({"Left":601,"Top":7,"Width":33,"Height":14,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":m_NewRecordPanel,"Text":"金额","Css":"FinanceLabeBold"});
        
        

        
        var m_NewAmount = new Controls.TextBox({"Left":639,"Top":5,"Width":130,"Height":22,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":m_NewRecordPanel,"Text":"","Css":"textbox","BorderWidth":1});
        
        

        
        var label55 = new Controls.Label({"Left":5,"Top":40,"Width":31,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_NewRecordPanel,"Text":"说明","Css":"FinanceLabeBold"});
        
        

        
        var m_NewComment = new Controls.TextBox({"Left":48,"Top":37,"Width":720,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":m_NewRecordPanel,"Text":"","Css":"textbox","BorderWidth":1});
        
        

        
        var m_BtnAddOK = new Controls.Button({"Left":673,"Top":68,"Width":95,"Height":26,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":m_NewRecordPanel,"Text":"添加记录","Css":"button_default"});
        
        
        
        m_BtnAddOK.OnClick.Attach(
            function(btn)
            {
                AddRecord();
            }
        )

        
        var control22 = new Controls.Control({"Left":5,"Top":8,"Width":775,"Height":100,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":m_Tab.GetPanel(1),"Text":"","Css":"control"});
        
        
        
        control22.OnResized.Attach(
            function(btn)
            {
                
            }
        )

        
        var m_CheckDate = new Controls.CheckBox({"Left":4,"Top":9,"Width":15,"Height":16,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"checkbox","BorderWidth":1});
        
        m_CheckDate.GetCheckBoxDom().checked = true;

        
        var label24 = new Controls.Label({"Left":23,"Top":8,"Width":16,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"从","Css":"FinanceLabeBold"});
        
        

        
        var m_SearchDateFrom_Config={"Left":43,"Top":4,"Width":180,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"control"};
        
                
        
        var m_SearchDateFrom = new DateControl(m_SearchDateFrom_Config);
        
        

        
        var label26 = new Controls.Label({"Left":231,"Top":8,"Width":16,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"到","Css":"FinanceLabeBold"});
        
        

        
        var m_SearchDateTo_Config={"Left":253,"Top":4,"Width":180,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"control"};
        
                
        
        var m_SearchDateTo = new DateControl(m_SearchDateTo_Config);
        
        {
            var now = new Date();
            m_SearchDateFrom.SetDate(new Date(now.getFullYear(), now.getMonth(), 1));
            m_SearchDateTo.SetDate(new Date(now.getFullYear(), now.getMonth(), GetDayCount(now.getYear(), now.getMonth() + 1)));
        }

        
        var m_CheckItem = new Controls.CheckBox({"Left":455,"Top":9,"Width":15,"Height":16,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"checkbox","BorderWidth":1});
        
        

        
        var label29 = new Controls.Label({"Left":475,"Top":8,"Width":62,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"收支项目","Css":"FinanceLabeBold"});
        
        

        
        var m_SearchFinanceItem_Config={"Left":542,"Top":5,"Width":225,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"control"};
        
                m_SearchFinanceItem_Config.Application = CurrentApplication;
        
        var m_SearchFinanceItem = new FinanceItemControl(m_SearchFinanceItem_Config);
        
        m_SearchFinanceItem.GetDom().tabIndex = -1;

        
        var m_CheckAmount = new Controls.CheckBox({"Left":4,"Top":41,"Width":16,"Height":16,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"checkbox","BorderWidth":1});
        
        

        
        var label34 = new Controls.Label({"Left":23,"Top":40,"Width":45,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"金额在","Css":"FinanceLabeBold"});
        
        

        
        var m_SearchAmountMin = new Controls.TextBox({"Left":70,"Top":37,"Width":140,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"textbox","BorderWidth":1});
        
        

        
        var m_SearchAmountMax = new Controls.TextBox({"Left":243,"Top":37,"Width":140,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"textbox","BorderWidth":1});
        
        

        
        var label37 = new Controls.Label({"Left":219,"Top":40,"Width":15,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"和","Css":"FinanceLabeBold"});
        
        

        
        var label38 = new Controls.Label({"Left":392,"Top":40,"Width":28,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"之间","Css":"FinanceLabeBold"});
        
        

        
        var m_CheckComment = new Controls.CheckBox({"Left":455,"Top":41,"Width":16,"Height":16,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"checkbox","BorderWidth":1});
        
        

        
        var label40 = new Controls.Label({"Left":474,"Top":39,"Width":63,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":control22,"Text":"说明包含","Css":"FinanceLabeBold"});
        
        

        
        var m_SearchComment = new Controls.TextBox({"Left":542,"Top":37,"Width":225,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":control22,"Text":"","Css":"textbox","BorderWidth":1});
        
        

        
        var m_BtnSearch = new Controls.Button({"Left":704,"Top":68,"Width":64,"Height":26,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":control22,"Text":"搜索","Css":"button_default"});
        
        
        
        m_BtnSearch.OnClick.Attach(
            function(btn)
            {
                Search();
            }
        )

        
        var m_SearchGrid_Config={"Left":1,"Top":110,"Width":782,"Height":419,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":m_Tab.GetPanel(1),"Text":"","Css":"control"};
        
                m_SearchGrid_Config.Application = CurrentApplication;
        
        var m_SearchGrid = new FinanceRecordGrid(m_SearchGrid_Config);
        
        

        
        var control44 = new Controls.Control({"Left":1,"Top":529,"Width":782,"Height":36,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Bottom,"Parent":m_Tab.GetPanel(1),"Text":"","Css":"Finance_Footer"});
        
        
        
        control44.OnResized.Attach(
            function(btn)
            {
                
            }
        )

        
        var m_SearchLabelStat = new Controls.Label({"Left":4,"Top":11,"Width":757,"Height":18,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":control44,"Text":"","Css":""});
        
        m_SearchLabelStat.GetDom().innerHTML="<div class='Finance_SearchLabelBalance'></div><div class='Finance_SearchLabelExpense'></div><div class='Finance_SearchLabelIncome'></div>";

        
        var toolbar2 = new Controls.Toolbar({"Left":1,"Top":1,"Width":786,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"toolbar","Items":[{"Text":"新建","Css":"Image22_NewProject","Command":"New"},{"Text":"打开","Css":"Image22_OpenFile","Command":"Open"}]});
        
        
        
        toolbar2.OnCommand.Attach(
            function(command)
            {
                if (command == "New")
                {
                    var config = {
                        InitPath: System.GetFullPath("Home"),
                        Title: "新建账本，请选择数据库文件路径..."
                    };
                
                    CommonDialog.SaveFile(config, This,
                    function(path)
                    {
                        This.Open(path, true);
                    });
                }
                else if (command == "Open")
                {
                    var config = {
                        InitPath: System.GetFullPath("Home"),
                        Title: "打开账本，请选择数据库文件路径..."
                    };
                    CommonDialog.OpenFile(config, This,
                    function callback(path)
                    {
                        This.Open(path[0]);
                    });
                }
            }
        )
    
    
        var m_Task = null;
        if(config.HasMinButton)
        {
            m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
            This.OnClosed.Attach(
                function()
                {
                    Taskbar.RemoveTask(m_Task);
                }
            );
        }
        m_NewItem.GetDom().tabIndex = -1;
        m_NewAmount.GetTextBoxDom().tabIndex = -1;
        m_NewComment.GetTextBoxDom().tabIndex = -1;
        m_NewRecordPanel.GetDom().tabIndex = -1;
        m_NewRecordPanel.GetDom().style.outline = "solid 0px black";
        
        HandleTabKey(m_NewItem.GetDom(), m_NewAmount.GetTextBoxDom());
        HandleTabKey(m_NewAmount.GetTextBoxDom(), m_NewComment.GetTextBoxDom());
        HandleTabKey(m_NewComment.GetTextBoxDom(), m_BtnAddOK.GetDom());
        HandleTabKey(m_BtnAddOK.GetDom(), m_NewItem.GetDom());
        
        System.AttachEvent(
        m_NewRecordPanel.GetDom(), "keydown",
        function(evt)
        {
            if (evt == undefined) evt = window.event;
            if (evt.keyCode == 13 && !evt.shiftKey && !evt.ctrlKey && !evt.altKey)
            {
                AddRecord();
                System.PreventDefault(evt);
                System.CancelBubble(evt);
            }
        });
        
        function FocusItemControl()
        {
            m_NewItem.GetDom().focus();
        }
        
        function FocusAmountTextBox()
        {
            m_NewAmount.GetTextBoxDom().focus();
        }
        
        function AddRecord()
        {
            var type = m_NewItem.GetSelectedType();
            if (type == 0)
            {
                CommonDialog.Alert("请选择收支类型！", This, FocusItemControl);
                return;
            }
        
            var amount = parseFloat(m_NewAmount.GetText());
            if (! (amount >= 0))
            {
                CommonDialog.Alert("请正确的金额！", This, FocusAmountTextBox);
                return;
            }
        
            var newRecord = {
                Key: Math.round((new Date()).getTime()),
                Type: type,
                Comment: m_NewComment.GetText(),
                Amount: amount,
                CreatedTime: m_NewDate.GetDate()
            };
        
            var cmd = String.format(InsertFinanceRecordCmd, type, TranslateCharForSQL(newRecord.Comment), newRecord.Amount, newRecord.CreatedTime.getTime() / 1000, (new Date()).getTime() / 1000, newRecord.Key);
        
            This.Waiting("正在添加记录...");
            SQLite.ExecuteNonQuery(
            function(ret)
            {
                This.Completed();
                if (ret.Result == "OK")
                {
                    m_RecordGrid.AppendRecord(newRecord);
        
                    m_NewComment.SetText("");
                    m_NewAmount.SetText("");
                }
            },
            function(ex)
            {
                This.Completed();
                CommonDialog.Alert(ex, This, FocusItemControl);
            },
            CurrentApplication.GetDbPath(), cmd);
        }
        
        function Search()
        {
            var where = [];
            if (m_CheckDate.GetCheckBoxDom().checked)
            {
                where.push(String.format("CreatedTime >= datetime('{0}','unixepoch','localtime') and CreatedTime <= datetime('{1}','unixepoch','localtime')", m_SearchDateFrom.GetDate().getTime() / 1000, m_SearchDateTo.GetDate().getTime() / 1000));
            }
        
            if (m_CheckItem.GetCheckBoxDom().checked)
            {
                where.push(String.format("fc.Type = {0}", m_SearchFinanceItem.GetSelectedType()));
            }
        
            if (m_CheckAmount.GetCheckBoxDom().checked)
            {
                var min = m_SearchAmountMin.GetText();
                var max = m_SearchAmountMax.GetText();
                if (min != "" && max != "")
                {
                    where.push(String.format("Amount >= {0} and Amount <= {1}", parseFloat(min), parseFloat(max)));
                }
                else if (min != "" && max == "")
                {
                    where.push(String.format("Amount >= {0}", parseFloat(min)));
                }
                else if (min == "" && max != "")
                {
                    where.push(String.format("Amount <= {0}", parseFloat(max)));
                }
            }
        
            if (m_CheckComment.GetCheckBoxDom().checked)
            {
                where.push(String.format("Comment like '%{0}%'", TranslateCharForSQL(m_SearchComment.GetText())));
            }
        
            var cmd = "select fc.Key as Key,fc.Type as Type,Comment,Amount,CreatedTime,LastRenewTime, Amount * (1- fi.Type *2) as TAmount from FinanceRecord fc, FinanceItem fi where fc.Type = fi.Key and " + where.join(" and ") + " order by CreatedTime";
        
            This.Waiting("正在查询...");
            SQLite.ExecuteQuery(
            function(ret)
            {
                This.Completed();
                if (ret.Result == "OK")
                {
                    m_SearchGrid.Clear();
                    try
                    {
                        var incomeSum = 0,
                        expenseSum = 0,
                        balanceSum = 0;
        
                        for (var i in ret.DataTable)
                        {
                            var row = ret.DataTable[i];
                            m_SearchGrid.AppendRecord(ret.DataTable[i]);
                            if (row.TAmount > 0) incomeSum += row.TAmount;
                            else expenseSum -= row.TAmount;
                            balanceSum += row.TAmount;
                        }
                        m_SearchLabelStat.GetDom().childNodes[2].innerHTML = String.format("收入合计:{0}", Round(incomeSum * 100) / 100);
                        m_SearchLabelStat.GetDom().childNodes[1].innerHTML = String.format("支出合计:{0}", Round(expenseSum * 100) / 100);
                        m_SearchLabelStat.GetDom().childNodes[0].innerHTML = String.format("结余:{0}", Round(balanceSum * 100) / 100);
                    }
                    catch(ex)
                    {
                        CommonDialog.Alert(ex, This);
                    }
                }
            },
            function(ex)
            {
                This.Completed();
                CommonDialog.Alert(ex, This);
            },
            CurrentApplication.GetDbPath(), cmd);
        }
        
        This.Open = function(path, raiseError)
        {
            if (raiseError == undefined) raiseError = false;
        
            This.Waiting("正在打开数据库...");
            try
            {
                SQLite.Create(function(ret)
                {
                    if (ret.Result == "OK" || ret.Result == "EXISTS")
                    {
                        m_DbPath = path;
                        SQLite.ExecuteQuery(
                        function(ret)
                        {
                            This.Completed();
                            try
                            {
                                m_FinanceItems = {
                                    Income: [],
                                    Expense: [],
                                    All: {},
                                    Shutcut: {}
                                };
        
                                for (var i in ret.DataTable)
                                {
                                    var row = ret.DataTable[i];
                                    if (row.Type == 0) m_FinanceItems.Income.push(row);
                                    else if (row.Type == 1) m_FinanceItems.Expense.push(row);
        
                                    m_FinanceItems.All[row.Key] = row;
                                    if (row.Shutcut.length > 0)
                                    {
                                        m_FinanceItems.Shutcut[row.Shutcut.charCodeAt(0)] = row;
                                    }
                                }
        
                                m_Tab.Select(0);
                                m_RecordGrid.Clear();
                                m_SearchGrid.Clear();
        
                                This.SetTitle(String.format("日常理财 [{0}]", System.Path.GetFileName(path)));
                            }
                            catch(ex)
                            {
                                CommonDialog.Alert(ex, This);
                            }
                        },
                        function(error)
                        {
                            This.Completed();
                            CommonDialog.Alert(error, This);
                        },
                        m_DbPath, SelectFinanceItemsCmd);
                    }
                    else
                    {
                        This.Completed();
                    }
                },
                function(error)
                {
                    This.Completed();
                    CommonDialog.Alert(error, This);
                },
                path, DDL, raiseError);
            }
            catch(ex)
            {
                This.Completed();
                CommonDialog.Alert(ex, This);
            }
        }
    }

}



function DateControl(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "Finance_DateControl";
    
    var width = config.Width, height = config.Height;
    config.Width=235;
    config.Height=24;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "DateControl"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    This.SetDate = function(date)
    {
        m_DDLYear.SetValue(date.getFullYear());
        m_DDLMonth.SetValue(date.getMonth() + 1);
        m_DDLDay.SetValue(date.getDate());
    }
    
    This.GetDate = function()
    {
        return new Date(m_DDLYear.GetValue(), m_DDLMonth.GetValue() - 1, m_DDLDay.GetValue(), 0, 0, 0);
    }
    
    This.GetDom().innerHTML = "<div class=\"DropDownList_Container\"></div>" + "<div class=\"DropDownList_Text\">年</div>" + "<div class=\"DropDownList_Container\"></div>" + "<div class=\"DropDownList_Text\">月</div>" + "<div class=\"DropDownList_Container\"></div>" + "<div class=\"DropDownList_Text\">日</div>";
    
    var m_DDLYear = new Controls.HtmlDropDownList(null, 45, 19);
    This.GetDom().childNodes[0].appendChild(m_DDLYear.GetDom());
    
    var m_DDLMonth = new Controls.HtmlDropDownList(null, 30, 19);
    This.GetDom().childNodes[2].appendChild(m_DDLMonth.GetDom());
    
    var m_DDLDay = new Controls.HtmlDropDownList(null, 40, 19);
    This.GetDom().childNodes[4].appendChild(m_DDLDay.GetDom());
    
    var now = new Date();
    
    for (var i = -10; i < 2; i++)
    {
        m_DDLYear.AddItem(now.getFullYear() + i, now.getFullYear() + i);
    }
    
    m_DDLYear.SetValue(now.getFullYear());
    
    for (var i = 1; i <= 12; i++)
    {
        m_DDLMonth.AddItem(i.toString(), i);
    }
    
    m_DDLMonth.SetValue(now.getMonth() + 1);
    
    function ResetDays(year, month)
    {
        var dayCount = GetDayCount(year, month);
        var current = IsNull(m_DDLDay.GetValue(), now.getDate());
    
        m_DDLDay.Clear();
        for (var i = 1; i <= dayCount; i++)
        {
            m_DDLDay.AddItem(i.toString(), i);
        }
    
        m_DDLDay.SetValue(current <= dayCount ? current: dayCount);
    }
    
    ResetDays(now.getFullYear(), now.getMonth() + 1);
    
    m_DDLYear.OnChanged.Attach(function()
    {
        ResetDays(m_DDLYear.GetValue(), m_DDLMonth.GetValue())
    });
    
    m_DDLMonth.OnChanged.Attach(function()
    {
        ResetDays(m_DDLYear.GetValue(), m_DDLMonth.GetValue())
    });

    This.Resize(width,height);
}


function FinanceItem_HtmlDropDownList(css, width, height, application)
{
    var This = this;

    var dom = document.createElement("DIV");
    dom.className = IsNull(css, "FinanceItemControl");

    dom.innerHTML = "<div></div>";

    var m_IsTreeVisible = false;

    function document_click(evt)
    {
        if (evt == undefined) evt = window.event;
        This.HideItemTree();
    }

    this.OnChanged = new Delegate();

    this.SwichItemTree = function()
    {
        if (m_IsTreeVisible) This.HideItemTree();
        else This.ShowItemTree();
    }

    this.ShowItemTree = function()
    {
        if (!m_IsTreeVisible)
        {
            var coordDesktop = System.GetClientCoord(Desktop.GetDom());
            var coord = System.GetClientCoord(dom);

            coord.X -= coordDesktop.X;
            coord.Y -= coordDesktop.Y;

            Desktop.EnterMove();

            var top = coord.Y + dom.offsetHeight;
            if (top + m_ItemTree.GetHeight() > Desktop.GetHeight())
            {
                top -= (m_ItemTree.GetHeight() + dom.offsetHeight);
                top += 1
            }
            else
            {
                top -= 1;
            }
            m_ItemTree.Move(coord.X, top);

            Desktop.AddControl(m_ItemTree);

            m_ItemTree.Expand(EmptyCallback, "/income");
            m_ItemTree.Expand(EmptyCallback, "/expense");

            m_IsTreeVisible = true;
            System.AttachEvent(document, "click", document_click);
        }
    }

    this.HideItemTree = function()
    {
        if (m_IsTreeVisible)
        {
            Desktop.RemoveControl(m_ItemTree);
            Desktop.LeaveMove();
            m_IsTreeVisible = false;
            System.DetachEvent(document, "click", document_click);
        }
    }

    this.GetDom = function()
    {
        return dom;
    }

    this.Resize = function(w, h)
    {
        dom.style.width = (w - 2) + "px";
        dom.style.height = (h - 2) + "px";

        m_ItemTree.Resize(w + 100, 320);
    }

    this.SetText = function(text)
    {
        dom.childNodes[0].innerHTML = System.ReplaceHtml(text);
    }

    this.GetText = function()
    {
        return dom.childNodes[0].innerHTML;
    }

    this.GetSelectedType = function()
    {
        var node = m_ItemTree.GetSelectedNode();
        return (node != null && node.GetTag() != null) ? node.GetTag().Key: 0;
    }

    this.Select = function(key)
    {
        var item = application.GetFinanceItemDetails(key);
        var text = item.Name;
        var root = (item.Type == 0 ? "/income": "/expense");
        var path = "";
        do
        {
            path += "/" + item.Key;
            item = application.GetFinanceItemDetails(item.Parent);
        } while ( item != null );

        m_ItemTree.Select(function(node)
        {
            if (node != null)
            {
                This.SetText(text);
                This.OnChanged.Call();
            }
        },
        root + path);
    }

    this.OnChanged = new Delegate();

    var m_ItemTree = new FinanceItemTree({
        Left: 0,
        Top: 0,
        Width: 200,
        Height: 200,
        Parent: null,
        AnchorStyle: Controls.AnchorStyle.Left | Controls.AnchorStyle.Top,
        BorderWidth: 1,
        Application: application
    });

    m_ItemTree.GetDom().style.zIndex = "2000000";

    function ItemTree_OnClick(node)
    {
        if (node.GetTag() != null)
        {
            This.HideItemTree();
            This.SetText(node.GetTag().Name);
            This.OnChanged.Call();
        }
    }

    m_ItemTree.OnClick.Attach(ItemTree_OnClick);

    this.Resize(width, height);

}

function FinanceItemControl(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "Finance_FinanceItemControl";
    
    var width = config.Width, height = config.Height;
    config.Width=123;
    config.Height=22;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "FinanceItemControl"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    var m_DDL = new FinanceItem_HtmlDropDownList(null, 40, 19, config.Application);
    This.GetDom().appendChild(m_DDL.GetDom());
    This.GetDom().style.outline = "solid 0px #FFFFFF";
    
    function OnResized()
    {
        m_DDL.Resize(This.GetClientWidth(), This.GetClientHeight());
    }
    
    This.OnResized.Attach(OnResized);
    
    This.SetText = m_DDL.SetText;
    
    This.GetSelectedType = function()
    {
        return m_DDL.GetSelectedType();
    }
    
    This.Select = function(key)
    {
        m_DDL.Select(key);
    }
    
    This.OnChanged = m_DDL.OnChanged;
    
    System.AttachEvent(This.GetDom(), "focus",
    function(evt)
    {
        m_DDL.ShowItemTree();
        m_DDL.GetDom().style.backgroundColor = "#FFFDDF";
        if (evt == undefined) evt = window.event;
        System.CancelBubble(evt);
    });
    
    System.AttachEvent(This.GetDom(), "click",
    function(evt)
    {
        m_DDL.ShowItemTree();
        m_DDL.GetDom().style.backgroundColor = "#FFFDDF";
        System.CancelBubble(evt);
    });
    
    System.AttachEvent(This.GetDom(), "blur",
    function(evt)
    {
        m_DDL.GetDom().style.backgroundColor = "#FFFFFF";
    });
    
    HandleTabKey(This.GetDom(), null,
    function()
    {
        m_DDL.HideItemTree();
    });
    
    System.AttachEvent(
    This.GetDom(), "keydown",
    function(evt)
    {
        if (evt == undefined) evt = window.event;
        var item = config.Application.GetFinanceItemByShutcut(evt.keyCode);
        if (item != null && !evt.shiftKey && !evt.ctrlKey && !evt.altKey)
        {
            This.Select(item.Key);
            System.PreventDefault(evt);
            System.CancelBubble(evt);
        }
        else if (evt.keyCode == 13 && !evt.shiftKey && !evt.ctrlKey && !evt.altKey)
        {
            m_DDL.SwichItemTree();
            System.PreventDefault(evt);
            System.CancelBubble(evt);
        }
    
    });

    This.Resize(width,height);
}




function FinanceRecordEditor(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "FinanceRecordEditor";
    
    var width = config.Width, height = config.Height;
    config.Width=717;
    config.Height=28;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "FinanceRecordEditor"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_Date_Config={"Left":154,"Top":2,"Width":172,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"control"};
    
            
    
    var m_Date = new DateControl(m_Date_Config);
    
    

    
    var m_FinanceItem_Config={"Left":4,"Top":3,"Width":142,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"control"};
    
            m_FinanceItem_Config.Application = config.Application;
    
    var m_FinanceItem = new FinanceItemControl(m_FinanceItem_Config);
    
    function FinanceItem_OnChanged()
    {
        m_Income.SetVisible(instance.GetFinanceItemDetails(m_FinanceItem.GetSelectedType()).Type == 0);
        m_Expense.SetVisible(instance.GetFinanceItemDetails(m_FinanceItem.GetSelectedType()).Type == 1);
    }
    
    m_FinanceItem.OnChanged.Attach(FinanceItem_OnChanged);
    

    
    var m_Income = new Controls.TextBox({"Left":334,"Top":3,"Width":92,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"textbox","BorderWidth":1});
    
    

    
    var m_Comment = new Controls.TextBox({"Left":534,"Top":3,"Width":77,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"textbox","BorderWidth":1});
    
    

    
    var m_Expense = new Controls.TextBox({"Left":434,"Top":3,"Width":92,"Height":22,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"textbox","BorderWidth":1});
    
    

    
    var m_LabelOpeLink = new Controls.Label({"Left":621,"Top":7,"Width":92,"Height":14,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"label"});
    
    m_LabelOpeLink.SetVisible(config.Mode == "Update");
    
    m_LabelOpeLink.GetDom().innerHTML = "<div class='OperationLink'>取消</div><div class='OperationLink'>更新</div>";
    
    m_LabelOpeLink.GetDom().childNodes[0].onclick = function()
    {
        m_Grid.Edit( - 1);
    }
    
    m_LabelOpeLink.GetDom().childNodes[1].onclick = function()
    {
        UpdateRecord();
    }

    
    var m_BtnAdd = new Controls.Button({"Left":648,"Top":1,"Width":64,"Height":26,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"添加","Css":"button_default"});
    
    m_BtnAdd.SetVisible(config.Mode == "New");
    
    m_BtnAdd.OnClick.Attach(
        function(btn)
        {
            AddRecord();
        }
    )
    

    var instance = config.Application;
    var m_Grid = config.Grid;
    
    var m_Record = null;
    
    This.Init = function()
    {
        m_FinanceItem.Select(7);
        m_Expense.SetText("");
        m_Income.SetText("");
        m_Date.SetDate(new Date());
        m_Comment.SetText("");
    }
    
    This.Reset = function(record)
    {
        m_FinanceItem.Select(record.Type);
        if (instance.GetFinanceItemDetails(record.Type).Type == 0)
        {
            m_Income.SetText(Round(record.Amount));
            m_Expense.SetText("");
        }
        else
        {
            m_Income.SetText("");
            m_Expense.SetText(Round(record.Amount));
        }
        m_Date.SetDate(record.CreatedTime);
        m_Comment.SetText(record.Comment);
        m_Record = record;
    }
    
    function FocusItemControl()
    {
        m_FinanceItem.GetDom().focus();
    }
    
    function FocusIncomeTextBox()
    {
        m_Income.GetTextBoxDom().focus();
    }
    
    function FocusExpenseTextBox()
    {
        m_Expense.GetTextBoxDom().focus();
    }
    
    function AddRecord()
    {
        var type = m_FinanceItem.GetSelectedType();
        if (type == 0)
        {
            CommonDialog.Alert("请选择收支类型！", This.GetTopContainer(), FocusItemControl);
            return;
        }
    
        var amount;
    
        if (instance.GetFinanceItemDetails(type).Type == 0)
        {
            amount = parseFloat(m_Income.GetText());
            if (! (amount >= 0))
            {
                CommonDialog.Alert("请正确的金额！", This.GetTopContainer(), FocusIncomeTextBox);
                return;
            }
        }
    
        if (instance.GetFinanceItemDetails(type).Type == 1)
        {
            amount = parseFloat(m_Expense.GetText());
            if (! (amount >= 0))
            {
                CommonDialog.Alert("请正确的金额！", This.GetTopContainer(), FocusExpenseTextBox);
                return;
            }
        }
    
        var newRecord = {
            Key: Math.round((new Date()).getTime()),
            Type: type,
            Comment: m_Comment.GetText(),
            Amount: amount,
            CreatedTime: m_Date.GetDate()
        };
    
        var cmd = String.format(InsertFinanceRecordCmd, type, TranslateCharForSQL(newRecord.Comment), newRecord.Amount, newRecord.CreatedTime.getTime() / 1000, (new Date()).getTime() / 1000, newRecord.Key);
    
        This.GetTopContainer().Waiting("正在添加记录...");
        SQLite.ExecuteNonQuery(
        function(ret)
        {
            This.GetTopContainer().Completed();
            if (ret.Result == "OK")
            {
                m_Grid.AppendRecord(newRecord);
    
                m_Comment.SetText("");
                m_Expense.SetText("");
                m_Income.SetText("");
            }
        },
        function(ex)
        {
            This.GetTopContainer().Completed();
            CommonDialog.Alert(ex, This.GetTopContainer(), FocusItemControl);
        },
        instance.GetDbPath(), cmd);
    }
    
    function UpdateRecord()
    {
        var type = m_FinanceItem.GetSelectedType();
        if (type == 0)
        {
            CommonDialog.Alert("请选择收支类型！", This.GetTopContainer());
            return;
        }
    
        var amount;
    
        if (instance.GetFinanceItemDetails(type).Type == 0)
        {
            amount = parseFloat(m_Income.GetText());
            if (! (amount >= 0))
            {
                CommonDialog.Alert("请正确的金额！", This.GetTopContainer());
                return;
            }
        }
    
        if (instance.GetFinanceItemDetails(type).Type == 1)
        {
            amount = parseFloat(m_Expense.GetText());
            if (! (amount >= 0))
            {
                CommonDialog.Alert("请正确的金额！", This.GetTopContainer());
                return;
            }
        }
    
        m_Record.Type = type;
        m_Record.Comment = m_Comment.GetText();
        m_Record.Amount = amount;
        m_Record.CreatedTime = m_Date.GetDate();
    
        var cmd = String.format(UpdateFinanceRecordCmd, type, TranslateCharForSQL(m_Record.Comment), m_Record.Amount, m_Record.CreatedTime.getTime() / 1000, (new Date()).getTime() / 1000, m_Record.Key);
    
        This.GetTopContainer().Waiting("正在更新记录...");
        SQLite.ExecuteNonQuery(
        function(ret)
        {
            This.GetTopContainer().Completed();
            if (ret.Result == "OK")
            {
                m_Grid.Refresh(m_Grid.IndexOf(m_Record));
                m_Grid.Edit( - 1);
            }
        },
        function(ex)
        {
            This.GetTopContainer().Completed();
            CommonDialog.Alert(ex, This.GetTopContainer());
        },
        instance.GetDbPath(), cmd);
    }
    
    m_FinanceItem.GetDom().tabIndex = -1;
    m_Income.GetDom().tabIndex = -1;
    m_Expense.GetDom().tabIndex = -1;
    m_Comment.GetDom().tabIndex = -1;
    
    HandleTabKey(m_FinanceItem.GetDom(), null,
    function()
    {
        var type = m_FinanceItem.GetSelectedType();
        if (type != 0 && instance.GetFinanceItemDetails(type).Type == 1)
        {
            m_Expense.GetTextBoxDom().focus();
        }
        else
    
        {
            m_Income.GetTextBoxDom().focus();
        }
    });
    HandleTabKey(m_Income.GetTextBoxDom(), m_Comment.GetTextBoxDom());
    HandleTabKey(m_Expense.GetTextBoxDom(), m_Comment.GetTextBoxDom());
    HandleTabKey(m_Comment.GetTextBoxDom(), m_FinanceItem.GetDom());
    
    System.AttachEvent(
    This.GetDom(), "keydown",
    function(evt)
    {
        if (evt == undefined) evt = window.event;
        if (evt.keyCode == 13 && !evt.shiftKey && !evt.ctrlKey && !evt.altKey)
        {
            if (config.Mode == "New")
            {
                AddRecord();
                System.PreventDefault(evt);
                System.CancelBubble(evt);
            }
        }
    });

    This.Resize(width,height);
}


function FinanceItemTree_DataSource(instance)
{
    this.GetSubNodes = function(callback, node)
    {
        var nodes = [];
        if (node == null)
        {
            nodes.push({
                Name: "income",
                Text: "收入项目",
                ImageCss: "Image16_Folder",
                HasChildren: true,
                Tag: null
            });
            nodes.push({
                Name: "expense",
                Text: "支出项目",
                ImageCss: "Image16_Folder",
                HasChildren: true,
                Tag: null
            });
            callback(nodes);
        }
        else if (node.GetFullPath() == "/income")
        {
            var items = instance.GetFinanceItems().Income;
            for (var i in items)
            {
                nodes.push({
                    Name: items[i].Key.toString(),
                    Text: String.format("{0} ({1})", items[i].Name, items[i].Shutcut),
                    ImageCss: "Image16_File",
                    HasChildren: false,
                    Tag: items[i]
                });
            }
            callback(nodes);
        }
        else if (node.GetFullPath() == "/expense")
        {
            var items = instance.GetFinanceItems().Expense;
            for (var i in items)
            {
                nodes.push({
                    Name: items[i].Key.toString(),
                    Text: String.format("{0} ({1})", items[i].Name, items[i].Shutcut),
                    ImageCss: "Image16_File",
                    HasChildren: false,
                    Tag: items[i]
                });
            }
            callback(nodes);
        }

    }
}

function FinanceItemTree(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "treeview";
config.DataSource = new FinanceItemTree_DataSource(config.Application);
    
    var width = config.Width, height = config.Height;
    config.Width=200;
    config.Height=200;

    Controls.TreeView.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "FinanceItemTree"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    

    function onclick(evt)
    {
        if (evt == undefined) evt = window.event;
        System.CancelBubble(evt);
    }
    
    System.AttachEvent(This.GetDom(), "click", onclick);
    
    This.Refresh(EmptyCallback);

    This.Resize(width,height);
}




function FinanceRecordGrid(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "FinanceRecordGrid";
    
    var width = config.Width, height = config.Height;
    config.Width=786;
    config.Height=520;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "FinanceRecordGrid"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_Header = new Controls.Control({"Left":0,"Top":0,"Width":786,"Height":28,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"FinanceRecordGridHeader"});
    
    
    
    m_Header.OnResized.Attach(
        function(btn)
        {
            
        }
    )

    
    var label4 = new Controls.Label({"Left":10,"Top":7,"Width":104,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_Header,"Text":"收入/支出项目","Css":"label"});
    
    

    
    var label5 = new Controls.Label({"Left":159,"Top":7,"Width":91,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_Header,"Text":"日期","Css":"label"});
    
    

    
    var label6 = new Controls.Label({"Left":334,"Top":7,"Width":80,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_Header,"Text":"收入金额","Css":"label"});
    
    

    
    var label7 = new Controls.Label({"Left":434,"Top":7,"Width":86,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_Header,"Text":"支出金额","Css":"label"});
    
    

    
    var label8 = new Controls.Label({"Left":535,"Top":7,"Width":97,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":m_Header,"Text":"说明","Css":"label"});
    
    

    
    var label9 = new Controls.Label({"Left":690,"Top":7,"Width":83,"Height":14,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":m_Header,"Text":"","Css":"label"});
    
    

    
    var m_Container = new Controls.Control({"Left":0,"Top":28,"Width":786,"Height":492,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"control"});
    
    m_Container.GetDom().style.overflowX = "hidden";
    m_Container.GetDom().style.overflowY = "auto";
    
    m_Container.OnResized.Attach(
        function(btn)
        {
            
        }
    )

    
    var m_Editor_Config={"Left":0,"Top":13,"Width":786,"Height":28,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":m_Container,"Text":"","Css":"control"};
    
            m_Editor_Config.Application = config.Application;
            m_Editor_Config.Mode = "Update";
            m_Editor_Config.Grid = this;
    
    var m_Editor = new FinanceRecordEditor(m_Editor_Config);
    
    m_Editor.SetVisible(false);
    m_Editor.GetDom().style.zIndex = '10';
    

    var m_Records = [];
    
    This.AppendRecord = function(record, edit)
    {
        var ctrl = new FinanceRecordCtrl({
            Left: 0,
            Top: m_Records.length * 28,
            Width: m_Container.GetClientWidth() - 0,
            Height: 28,
            Parent: m_Container,
            AnchorStyle: Controls.AnchorStyle.Left | Controls.AnchorStyle.Right | Controls.AnchorStyle.Top,
            Application: config.Application,
            Record: record,
            Grid: This
        });
    
        ctrl.GetDom().style.backgroundColor = (m_Records.length % 2) != 0 ? "#FAFAFA": "#FFFFFF";
        ctrl.GetDom().scrollIntoView();
        m_Records.push(ctrl);
        if (edit != undefined && edit) This.Edit(m_Records.length - 1);
    }
    
    This.Edit = function(index)
    {
        if (index >= 0)
        {
            m_Editor.Move(0, index * 28);
            m_Editor.Reset(m_Records[index].GetRecord());
            m_Editor.SetVisible(true);
        }
        else
        {
            m_Editor.SetVisible(false);
        }
    }
    
    This.Refresh = function(index)
    {
        if (index >= 0)
        {
            m_Records[index].Refresh();
        }
    }
    
    This.Delete = function(index)
    {
        if (index >= 0 && index < m_Records.length)
        {
            m_Container.RemoveControl(m_Records[index]);
            m_Records.splice(index, 1);
            for (; index < m_Records.length; index++)
            {
                m_Records[index].Move(m_Records[index].GetLeft(), m_Records[index].GetTop() - 28);
                m_Records[index].GetDom().style.backgroundColor = (index % 2) != 0 ? "#FAFAFA": "#FFFFFF";
            }
        }
    }
    
    This.Clear = function()
    {
        for (var i = 0; i < m_Records.length; i++)
        {
            m_Container.RemoveControl(m_Records[i]);
        }
        m_Records = [];
    }
    
    This.IndexOf = function(record)
    {
        for (var i = 0; i < m_Records.length; i++)
        {
            if (m_Records[i].GetRecord().Key == record.Key) return i;
        }
        return - 1;
    }

    This.Resize(width,height);
}




function FinanceRecordCtrl(config)
{
    var This = this;
    var OwnerForm = this;
    
config.Css = "FinanceRecordCtrl";
    
    var width = config.Width, height = config.Height;
    config.Width=804;
    config.Height=28;

    Control.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "FinanceRecordCtrl"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var m_LabelItem = new Controls.Label({"Left":8,"Top":7,"Width":137,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"LabelItem"});
    
    

    
    var m_LabelDate = new Controls.Label({"Left":160,"Top":7,"Width":166,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"LabelDate"});
    
    

    
    var m_LabelIncome = new Controls.Label({"Left":334,"Top":7,"Width":92,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"LabelIncome"});
    
    

    
    var m_LabelExpense = new Controls.Label({"Left":434,"Top":7,"Width":92,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"LabelExpense"});
    
    

    
    var m_LabelOperation = new Controls.Label({"Left":708,"Top":7,"Width":92,"Height":14,"AnchorStyle":Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"label"});
    
    m_LabelOperation.GetDom().innerHTML = "<div class='OperationLink'>编辑</div><div class='OperationLink'>删除</div>";
    
    m_LabelOperation.GetDom().childNodes[0].onclick = function()
    {
        m_Grid.Edit(m_Grid.IndexOf(record));
    }
    
    m_LabelOperation.GetDom().childNodes[1].onclick = function()
    {
        DeleteRecord();
    }

    
    var m_LabelComment = new Controls.Label({"Left":534,"Top":7,"Width":166,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"LabelComment"});
    
    
    

    var instance = config.Application;
    var m_Grid = config.Grid;
    var record = config.Record;
    
    This.GetRecord = function()
    {
        return record;
    }
    
    This.Refresh = function()
    {
        if (record != undefined)
        {
            m_LabelItem.SetText(instance.GetFinanceItemDetails(record.Type).Name);
            m_LabelDate.SetText(DateToString(record.CreatedTime));
            if (instance.GetFinanceItemDetails(record.Type).Type == 0)
            {
                m_LabelIncome.SetText(Round(record.Amount));
                m_LabelExpense.SetText("");
            }
            else
            {
                m_LabelIncome.SetText("");
                m_LabelExpense.SetText(Round(record.Amount));
            }
            m_LabelComment.SetText(record.Comment);
        }
    }
    
    function DeleteRecord()
    {
        var cmd = String.format(DeleteFinanceRecordCmd, record.Key);
    
        This.GetTopContainer().Waiting("正在删除记录...");
        SQLite.ExecuteNonQuery(
        function(ret)
        {
            This.GetTopContainer().Completed();
            if (ret.Result == "OK")
            {
                m_Grid.Edit( - 1);
                m_Grid.Delete(m_Grid.IndexOf(record));
            }
        },
        function(ex)
        {
            This.GetTopContainer().Completed();
            CommonDialog.Alert(ex, This.GetTopContainer());
        },
        instance.GetDbPath(), cmd);
    }
    
    This.Refresh();

    This.Resize(width,height);
}

