if(!TESTING)
{
    System.Link("StyleSheet", Module.GetResourceUrl("Notepad.css"), "text/css");
}

var Controls = null, CommonDialog = null, IO = null, Communication = null, Management = null;
var Window = null, Control = null, RichEditor = null;

function init(completeCallback, errorCallback)
{
    function LoadModulesComplete()
    {
        IO = System.GetModule("IO.js");
        Communication = System.GetModule("Communication.js");
        Management = System.GetModule("Management.js");
        Controls = System.GetModule("Controls.js");
        CommonDialog = System.GetModule("CommonDialog.js");

        Window = System.GetModule("Window.js").Window;
        Control = System.GetModule("Controls.js").Control;
        
        RichEditor = System.GetModule("RichEditor.js").RichEditor;

        _init(completeCallback, errorCallback);
    }

    System.LoadModules(
        LoadModulesComplete,
        errorCallback,
        ["IO.js", "Communication.js", "Management.js", "Window.js", "Controls.js", "CommonDialog.js", "RichEditor.js"]
    );
}

function _init(completeCallback, errorCallback)
{
    try
    {
        //初始化代码，初始化完成后必须调用completeCallback;
        IO.File.Open(
        function(file)
        {
            Config = (file.Content == "" ? DefaultConfig: System.ParseJson(file.Content));
            PreConfig = System.RenderJson(Config);
            completeCallback();
        },
        function(ex)
        {
            errorCallback(ex);
        },
        ConfigFilePath, true, "utf-8");
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}



function dispose(completeCallback, errorCallback)
{
    _dispose(completeCallback, errorCallback);
}

function _dispose(completeCallback, errorCallback)
{
    try
    {
        //卸载代码，卸载完成后必须调用completeCallback;
        var cs = System.RenderJson(Config);
        if (PreConfig != cs)
        {
            IO.File.Create(
            function()
            {
                completeCallback();
            },
            function(ex)
            {
                errorCallback(ex);
            },
            ConfigFilePath, cs, "utf-8");
        }
        else
        {
            completeCallback();
        }
    }
    catch (ex)
    {
        errorCallback(new Exception(ex.mame, ex.message));
    }
}

//共享全局变量和函数，在此定义的变量和函数将由该应用程序的所有实例共享
var DefaultConfig={
    FontFamily:"宋体",FontSize:12,FontStyle:0
};
var PreConfig={};
var Config={};

var ConfigFilePath=System.GetFullPath(String.format("Config/6DE1A9E6-D657-4d1f-B412-213640B9AAFA.conf"));

var LastestPath=System.GetFullPath("Home");

function OptionListBoxItem(text,value)
{
    this.GetValue=function(){return value;}
    this.GetText=function(){return text;}
}

function Application()
{
    var CurrentApplication = this;
    var m_MainForm = null;
    
    //应用程序全局对象;
    var m_FileName = null;
    var m_LatestContent = null;
    
    this.BeforeUnload = function()
    {
        return '=============================\n' + '\n' + '  离开页面前请先退出记事本！ \n' + '\n' + '=============================\n'
    }    

    this.Start = function(baseUrl)
    {
        //应用程序入口;
        if (arguments[1] != undefined) m_FileName = arguments[1];
        
        m_MainForm = new MainForm();
        m_MainForm.OnClosed.Attach(
            function()
            {
                CurrentApplication.Dispose();
            }
        );
        m_MainForm.Resize(800, 600);
        m_MainForm.MoveEx("center", 0, 0);
        m_MainForm.Show(true);
        
        if(m_FileName!=null) m_MainForm.Open(m_FileName);
    }

    this.Terminate = function(completeCallback, errorCallback)
    {
        try
        {
            //应用程序终止，退出系统时用系统调用;
            completeCallback();
        }
        catch (ex)
        {
            errorCallback(new Exception(ex.mame, ex.message));
        }
    }
    function MainForm()
    {
        var This = this;
        var OwnerForm = this;
        
        var config = {"Left":74,"Top":33,"Width":433,"Height":336,"AnchorStyle":Controls.AnchorStyle.Left | Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":true,"HasMaxButton":true,"Resizable":true,"Title":{"InnerHTML":"记事本"}};
        config.OnClose = function()
        {
            if (IsChanged())
            {
                var confirmDialog = new CloseConfirmDialog(m_FileName == null ? null: System.Path.GetFileName(m_FileName));
                function confirm_callback()
                {
                    if (confirmDialog.DialogResult == "Yes")
                    {
                        This.Save(m_FileName,
                        function()
                        {
                            This.Close();
                        });
                    }
                    else if (confirmDialog.DialogResult == "No")
                    {
                        This.Close();
                    }
                }
                confirmDialog.ShowDialog(This, 'center', 0, 0, confirm_callback);
            }
            else
            {
                This.Close();
            }
        }
    
        Window.call(This, config);
    
        var Base = {
            GetType: This.GetType,
            is: This.is
        };
    
        This.GetType = function() { return "MainForm"; }
        This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
        
        var toolbar1 = new Controls.Toolbar({"Left":2,"Top":2,"Width":417,"Height":24,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"toolbar","Items":[{"Text":"打开","Css":"Notepad_OpenButton","Command":"Open"},{"Text":"保存","Css":"Notepad_SaveButton","Command":"Save"},{"Text":"另存为","Css":"Notepad_SaveAsButton","Command":"SaveAs"},{"Text":"取消自动换行","Css":"Notepad_OpenButton","Command":"Wrap"},{"Text":"选项","Css":"Notepad_OptionButton","Command":"Option"}]});
        
        toolbar1.CreateControl({
            Type: "DropDownList",
            Width: 100
        });
        
        toolbar1.GetControl(5).AddItem("utf-8", "utf-8");
        toolbar1.GetControl(5).AddItem("gb2312", "gb2312");
        toolbar1.GetControl(5).AddItem("big5", "big5");
        toolbar1.GetControl(5).AddItem("unicode", "unicode");
        toolbar1.GetControl(5).SetText("utf-8");
        
        toolbar1.GetControl(5).OnChanged.Attach(
        function()
        {    if (!IsChanged() && m_FileName != null)
            {
                function callback()
                {
                    This.Open(m_FileName);
                }
                CommonDialog.Confirm("是否重新加载文件？", This, callback);
            }
        
        });
        
        if (System.GetBrowser() != "IE") toolbar1.SetButtonVisible(3, false);
        
        toolbar1.OnCommand.Attach(
            function(command)
            {
                if (command == "Open")
                {
                    This.Open();
                }
                else if (command == "Save")
                {
                    This.Save(m_FileName);
                }
                else if (command == "SaveAs")
                {
                    This.Save();
                }
                else if (command == "Wrap")
                {
                    if (editor.GetTextAreaDom().wrap == "soft")
                    {
                        toolbar1.SetButtonText(3, "自动换行");
                        editor.GetTextAreaDom().wrap = "off"
                    }
                    else
                    {
                        toolbar1.SetButtonText(3, "取消自动换行");
                        editor.GetTextAreaDom().wrap = "soft";
                    }
                    editor.SetText(editor.GetText());
                }
                else if (command == "Option")
                {
                    var optionForm = new OptionForm(This);
                    function callback()
                    {
                    }
                    optionForm.ShowDialog(This, 'center', 0, 0, callback);
                }
            }
        )

        
        var editor = new Controls.TextArea({"Left":2,"Top":28,"Width":417,"Height":277,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"textarea","BorderWidth":1});
        
        function editor_onkeydown(evt)
        {
            if (evt == undefined) evt = window.event;
            if (evt.keyCode == 9 && !evt.shiftKey && !evt.ctrlKey && !evt.altKey)
            {
                var scrollTop = this.scrollTop;
                if (this.setSelectionRange)
                {
                    var sS = this.selectionStart;
                    var sE = this.selectionEnd;
                    this.value = this.value.substring(0, sS) + "    " + this.value.substr(sE);
                    this.setSelectionRange(sS + 4, sS + 4);
                    this.focus();
                    System.PreventDefault(evt);
                }
                else if (this.createTextRange)
                {
                    document.selection.createRange().text = "    ";
                    this.focus();
                    System.PreventDefault(evt);
                }
                this.scrollTop = scrollTop;
            }
        }
        
        System.AttachEvent(editor.GetTextAreaDom(), "keydown", editor_onkeydown);
    
    
        var m_Task = null;
        if(config.HasMinButton)
        {
            m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
            This.OnClosed.Attach(
                function()
                {
                    Taskbar.RemoveTask(m_Task);
                }
            );
        }
        function IsChanged()
        {
            return (editor.GetText() != "" && editor.GetText() != m_LatestContent);
        }
        
        This.SetEditorStyle = function(family, size, style)
        {
            var editor = This.GetEditor();
            editor.style.fontFamily = family;
            editor.style.fontSize = size + 'px';
        
            switch (style)
            {
            case "0":
                {
                    editor.style.fontStyle = "normal";
                    editor.style.fontWeight = "normal";
                    break;
                }
            case "1":
                {
                    editor.style.fontStyle = "italic";
                    editor.style.fontWeight = "normal";
                    break;
                }
            case "2":
                {
                    editor.style.fontStyle = "normal";
                    editor.style.fontWeight = "bold";
                    break;
                }
            case "3":
                {
                    editor.style.fontStyle = "italic";
                    editor.style.fontWeight = "bold";
                    break;
                }
            }
        
            Config.FontFamily = family;
            Config.FontSize = size;
            Config.FontStyle = style;
        }
        
        This.Open = function(fileName)
        {
            function openfile_callback(paths)
            {
                function open_success(file)
                {
                    m_FileName = paths[0];
                    LastestPath = System.Path.GetDirectoryName(m_FileName);
                    This.Completed();
                    editor.SetText(file.Content);
                    m_LatestContent = editor.GetText();
                }
        
                function open_error(ex)
                {
                    This.Completed();
                    CommonDialog.Alert(ex.Message, This);
                }
        
                This.Waiting("正在打开文件...");
                try
                {
                    if (paths.length > 0)
                    {
                        IO.File.Open(open_success, open_error, paths[0], false, toolbar1.GetControl(5).GetText());
                    }
                }
                catch(ex)
                {
                    CommonDialog.Alert(ex.Message, This);
                    This.Completed();
                }
            }
        
            var of_config = {
                InitPath: LastestPath
            };
        
            if (fileName == undefined || fileName == null)
            {
                if (IsChanged())
                {
                    var confirmDialog = new CloseConfirmDialog(m_FileName == null ? null: System.Path.GetFileName(m_FileName));
                    function confirm_callback()
                    {
                        if (confirmDialog.DialogResult == "Yes")
                        {
                            This.Save(m_FileName,
                            function()
                            {
                                CommonDialog.OpenFile(of_config, This, openfile_callback);
                            });
        
                        }
                        else if (confirmDialog.DialogResult == "No")
                        {
                            CommonDialog.OpenFile(of_config, This, openfile_callback);
                        }
                    }
                    confirmDialog.ShowDialog(This, 'center', 0, 0, confirm_callback);
                }
                else
                {
                    CommonDialog.OpenFile(of_config, This, openfile_callback);
                }
            }
            else
            {
                openfile_callback([fileName]);
            }
        }
        
        This.Save = function(fileName, callback)
        {
            function savefile_callback(path)
            {
                function save_success()
                {
                    m_FileName = path;
                    LastestPath = System.Path.GetDirectoryName(m_FileName);
                    m_LatestContent = editor.GetText();
                    This.Completed();
                    if (callback != undefined) callback();
                }
        
                function save_error(ex)
                {
                    This.Completed();
                    CommonDialog.Alert(ex.Message, This);
                    if (callback != undefined) callback();
                }
        
                This.Waiting("正在保存文件...");
                try
                {
                    if (path != null)
                    {
                        IO.File.Create(save_success, save_error, path, editor.GetText(),toolbar1.GetControl(5).GetText());
                    }
                }
                catch(ex)
                {
                    CommonDialog.Alert(ex.Message, This);
                    This.Completed();
                }
            }
        
            var sf_config = {
                InitPath: LastestPath
            };
        
            if (fileName == undefined || fileName == null) CommonDialog.SaveFile(sf_config, This, savefile_callback);
            else savefile_callback(fileName);
        }
        
        This.GetEditor = function()
        {
            return editor.GetTextAreaDom();
        }
        
        This.SetEditorStyle(Config.FontFamily,Config.FontSize,Config.FontStyle);
        
    }

}
function CloseConfirmDialog()
{
    var This = this;
    var OwnerForm = this;
    
    var config = {"Left":44,"Top":17,"Width":259,"Height":145,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":false,"HasMaxButton":false,"Resizable":false,"Title":{"InnerHTML":"提示"}};
    

    Window.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "CloseConfirmDialog"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var label1 = new Controls.Label({"Left":65,"Top":10,"Width":170,"Height":64,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Right|Controls.AnchorStyle.Top|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"","Css":"Notepad_ConfirmText"});
    
    if (arguments[0] == undefined || arguments[0] == null)
    {
        label1.SetText("内容已改变。<br/>是否保存？");
    }
    else
    {
        label1.SetText(String.format("文件 \"{0}\" 的内容已改变。<br/>是否保存？", arguments[0]));
    }

    
    var control2 = new Controls.Control({"Left":20,"Top":14,"Width":32,"Height":32,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"Notepad_ConfirmIcon"});
    
    
    
    control2.OnResized.Attach(
        function(btn)
        {
            
        }
    )

    
    var button3 = new Controls.Button({"Left":18,"Top":80,"Width":64,"Height":26,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"是","Css":"button_default"});
    
    
    
    button3.OnClick.Attach(
        function(btn)
        {
            This.DialogResult="Yes";
            This.Close();
        }
    )

    
    var button4 = new Controls.Button({"Left":91,"Top":80,"Width":64,"Height":26,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"否","Css":"button"});
    
    
    
    button4.OnClick.Attach(
        function(btn)
        {
            This.DialogResult="No";
            This.Close();
        }
    )

    
    var button5 = new Controls.Button({"Left":164,"Top":80,"Width":64,"Height":26,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Bottom,"Parent":This,"Text":"取消","Css":"button"});
    
    
    
    button5.OnClick.Attach(
        function(btn)
        {
            This.DialogResult="Cancel";
            This.Close();
        }
    )
    

    var m_Task = null;
    if(config.HasMinButton)
    {
        m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
        This.OnClosed.Attach(
            function()
            {
                Taskbar.RemoveTask(m_Task);
            }
        );
    }
    This.DialogResult="Cancel";
    This.SetAcceptButton(button3);
    This.SetCancelButton(button5);
}

function OptionForm()
{
    var This = this;
    var OwnerForm = this;
    
    var config = {"Left":10,"Top":11,"Width":371,"Height":268,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":null,"Css":"window","BorderWidth":6,"HasMinButton":false,"HasMaxButton":false,"Resizable":false,"Title":{"InnerHTML":"选项"}};
    

    Window.call(This, config);

    var Base = {
        GetType: This.GetType,
        is: This.is
    };

    This.GetType = function() { return "OptionForm"; }
    This.is = function(type) { return type == This.GetType() ? true : Base.is(type); }
    
    var tab2 = new Controls.SimpleTabControl({"Left":1,"Top":1,"Width":357,"Height":236,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":This,"Text":"","Css":"simple_tab","Tabs":[{"Text":"字体","Width":80,"ID":"ID000037","IsSelected":true}],"BorderWidth":1});
    
    
    
    tab2.OnSelectedTab.Attach(
        function(index,preIndex)
        {
            
        }
    )

    
    var label4 = new Controls.Label({"Left":8,"Top":9,"Width":60,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":tab2.GetPanel(0),"Text":"字体：","Css":"label"});
    
    

    
    var lbFamily = new Controls.ListBox({"Left":8,"Top":28,"Width":115,"Height":132,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":tab2.GetPanel(0),"Text":"","Css":"Notepad_ListBox"});
    
    lbFamily.AddItem(new OptionListBoxItem("宋体","宋体"));
    lbFamily.AddItem(new OptionListBoxItem("雅黑","雅黑"));
    lbFamily.AddItem(new OptionListBoxItem("Courier New","Courier New"));
    lbFamily.AddItem(new OptionListBoxItem("Times New Roman","Times New Roman"));
    lbFamily.SetValue(Config.FontFamily);

    
    var label6 = new Controls.Label({"Left":137,"Top":9,"Width":60,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":tab2.GetPanel(0),"Text":"字形：","Css":"label"});
    
    

    
    var label7 = new Controls.Label({"Left":267,"Top":9,"Width":60,"Height":14,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":tab2.GetPanel(0),"Text":"大小：","Css":"label"});
    
    

    
    var lbStyle = new Controls.ListBox({"Left":137,"Top":28,"Width":115,"Height":132,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":tab2.GetPanel(0),"Text":"","Css":"Notepad_ListBox"});
    
    lbStyle.AddItem(new OptionListBoxItem("常规",0));
    lbStyle.AddItem(new OptionListBoxItem("斜体",1));
    lbStyle.AddItem(new OptionListBoxItem("加粗",2));
    lbStyle.AddItem(new OptionListBoxItem("粗斜体",3));
    
    lbStyle.SetValue(Config.FontStyle);

    
    var lbSize = new Controls.ListBox({"Left":266,"Top":28,"Width":77,"Height":132,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":tab2.GetPanel(0),"Text":"","Css":"Notepad_ListBox"});
    
    for (var i = 8; i <= 28; i += 2)
    {
        lbSize.AddItem(new OptionListBoxItem(i, i));
    }
    for (var i = 36; i <= 72; i += 12)
    {
        lbSize.AddItem(new OptionListBoxItem(i, i));
    }
    
    lbSize.SetValue(Config.FontSize);

    
    var button11 = new Controls.Button({"Left":279,"Top":172,"Width":64,"Height":26,"AnchorStyle":Controls.AnchorStyle.Left|Controls.AnchorStyle.Top,"Parent":tab2.GetPanel(0),"Text":"确定","Css":"button_default"});
    
    
    
    button11.OnClick.Attach(
        function(btn)
        {
            var editor = m_MainForm.SetEditorStyle(lbFamily.GetValue(), lbSize.GetValue(), lbStyle.GetValue());
            
            This.Close();
        }
    )
    

    var m_Task = null;
    if(config.HasMinButton)
    {
        m_Task=Taskbar.AddTask(This,IsNull(config.Title.InnerHTML,""));
        This.OnClosed.Attach(
            function()
            {
                Taskbar.RemoveTask(m_Task);
            }
        );
    }
    var m_MainForm=arguments[0];
}

